<?php
class AgenciesController extends AppController {
	public $components = array('DaiSharedFunction');
  
	public $pagination_fields = array(
		'Agency.id',
		'Agency.name',
		'Agency.address',
		'Agency.contact_person',
		'Agency.contact_no',
		'Agency.email_address',
		'Agency.remarks',
		'Agency.lock_flag',
		'Agency.status'
	);
	
	public $pagination_fields_reorder = array(
		'Agency.id',
		'Agency.name',
		'Agency.address',
		'Agency.contact_person',
		'Agency.contact_no',
		'Agency.email_address',
		'Agency.remarks',
		'Agency.lock_flag_name',
		'Agency.status_name'
	);
			
	public $report_fields = array(
		'Agency.id',
		'Agency.name',
		'Agency.address',
		'Agency.contact_person',
		'Agency.contact_no',
		'Agency.email_address',
		'Agency.remarks',
		'Agency.lock_flag',
		'Agency.status'
	);
	
	public $report_fields_reorder = array(
		'Agency.name',
		'Agency.address',
		'Agency.contact_person',
		'Agency.contact_no',
		'Agency.email_address',
		'Agency.remarks',
		'Agency.lock_flag_name',
		'Agency.status_name'
	);
	
	public $default_contain = array(
		
	);
	
	public $default_order = array('Agency.name' => 'ASC');
	
	public $default_conditions = array();
	
	public $default_recursive = -1;
	
	public $default_header_data = array(
		'Name',
		'Address',
		'Contact Person',
		'Contact No',
		'Email Address',
		'Remarks',
		'Locked',
		'Status'
	);
	
	public $return_address = array('controller' => 'agencies', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_success = array('controller' => 'agencies', 'action' => 'view_agency', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_add_no_layout = array('controller' => 'agencies', 'action' => 'add_agency', 'prefix' => 'bidding', 'bidding' => true, '?' => array('no-layout' => 1));
	
	public $search_form_name = 'SearchAgency';
	public $search_form_name_for_project = 'SearchProject';
	
	public $singular_alert_object = 'Agency';
	
	public $plural_alert_object = 'Agencies';
	
	public $singular_set_name = 'agency';
	
	public $plural_set_name = 'agencies';
	
	public function bidding_index() { 
		$records = $this->get_agencies();
	 
		if(!$records) {
			$this->Alert->display('no_records_found');
		}
		
		$this->set($this->plural_set_name, $records);
	}
	
	public function bidding_export_csv_agency() { 
		
		$records = $this->get_agencies(
			array(),
			false
		);
		
		$this->Export->export_csv(array(
			'title' => 'Agency List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_excel_agency() { 
		
		$records = $this->get_agencies(
			array(),
			false
		);
		
		$this->Export->export_excel(array(
			'title' => 'Agency List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_pdf_agency() { 
		$records = $this->get_agencies(
			array(),
			false
		);
		
		$this->Export->export_pdf(array(
			'title' => 'Agency List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}

	public function bidding_add_agency() { 
		$this->DaiSharedFunction->add_record_with_no_layout_option(
			$this->Agency, 
			$this->return_address,
			$this->return_address_on_add_no_layout
		); 
		
	}
	
	public function bidding_get_list() {
		return $this->DaiSharedFunction->get_list(
			$this->Agency
		);
	}
	
	public function bidding_edit_agency($id = null) { 
		$this->DaiSharedFunction->edit_and_lock_record(
			$this->Agency, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_override_password($id = null) { 
		$this->DaiSharedFunction->override_record(
			$this->Agency, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_change_password(){ 
		$this->Agency->attach_model();
		
		if($this->request->is('put') || $this->request->is('post')) {
			if($this->Agency->change_password($this->request->data)) {
				$this->Alert->display('success_change_password');
				return $this->redirect(array('controller' => 'home', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
			else {
				$this->Alert->display('failed_change_password');
			}
		}
	}
	
	public function bidding_view_agency($id = null) { 
		if($id){
			$agency = $this->Agency->find_by_id($id);
			if(!$agency) {
				$this->Alert->display('no_records_found');
				$this->redirect(array('controller' => 'agencies', 'action' => 'index', 'prefix' => 'im', 'im' => true));				
			}
			
			$project_list = $this->get_project_list_for_agency(
				array('Project.agency_id' => $agency['Agency']['id'])
			);
			
			
			
			$this->loadModel('ProjectStatus');
			$project_status_list = $this->ProjectStatus->get_list();
			$this->set('project_status_list', $project_status_list);
			$this->set('agency', $agency);
			$this->set('project_list', $project_list);
		}
		else {
			$this->Alert->display('no_records_found');
			$this->redirect(array('controller' => 'agencies', 'action' => 'index', 'prefix' => 'im', 'im' => true));
		}
		
		
		// $this->set('agency', $agency);
	}
	
	public function bidding_enable_agency($id = null) {  
		$this->DaiSharedFunction->enable_record(
			$this->Agency, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_disable_agency($id = null) { 
		
		$this->DaiSharedFunction->disable_record(
			$this->Agency, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_delete_agency($id = null) { 
		$this->DaiSharedFunction->delete_record(
			$this->Agency, 
			$this->return_address,
			$id
		);
	}
	
	private function get_agencies($submitted_conditions = array(), $is_paginated = true) {
		$search_form_name = $this->search_form_name;
		
		$this->default_conditions = array();
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.name', $search_form_name, 'agency_name')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.address', $search_form_name, 'address')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.contact_person', $search_form_name, 'contact_person')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.contact_no', $search_form_name, 'contact_no')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.email_address', $search_form_name, 'email_address'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.remarks', $search_form_name, 'remarks'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Agency.lock_flag', $search_form_name, 'locked'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_status('Agency.status', $search_form_name, 'status'));
 
		return $this->DaiSharedFunction->get_records(
			$this->Agency, 
			$this->pagination_fields,
			$this->pagination_fields_reorder,
			$this->report_fields,
			$this->report_fields_reorder,
			$this->default_contain,
			$this->default_conditions,
			$this->default_order,
			$this->default_recursive,
			$is_paginated
		); 
	}
	
	private function get_project_list_for_agency($submitted_conditions = array(), $is_paginated = true) {
		$search_form_name_for_project = $this->search_form_name_for_project;
		$this->loadModel('Project');
		// debug($submitted_conditions);
		// exit();
		$this->default_conditions = array(); 
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions);
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.name', $search_form_name_for_project, 'agency_name')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Project.name', $search_form_name_for_project, 'project_name'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_range('Project.deadline_of_submission', $search_form_name_for_project, 'deadline_of_submission'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_range('Project.pre_bidding_date', $search_form_name_for_project, 'pre_bidding_date'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_equivalent('Project.target_delivery_date', $search_form_name_for_project, 'target_delivery_date'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Project.project_status_id', $search_form_name_for_project, 'project_status'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Project.name_of_company_won', $search_form_name_for_project, 'name_of_company_won'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Project.lock_flag', $search_form_name_for_project, 'locked'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_status('Project.status', $search_form_name_for_project, 'status'));

		if (isset($this->request['url']['sort']) && $this->request['url']['direction']) {
			$default_order = $this->request['url']['sort'] . ' ' . $this->request['url']['direction'];
		}
		else if(isset($this->request['named']['sort']) && $this->request['named']['direction']) {
			$default_order = $this->request['named']['sort'] . ' ' . $this->request['named']['direction'];
		}
		// debug($this->default_conditions);
		// exit();
		$default_contain = array(
			'Agency',
			'ProjectStatus'
		);
		
		if ($is_paginated) {
			$this->Paginator->settings = array(
				'conditions' => $this->default_conditions,
				'contain' => $default_contain,
				// 'order' => $default_order,
				'recursive' => -1
			);
			
			try {
				$project_list = $this->Paginator->paginate('Project');
				
				$project_list = reorder(array(
					'fields' => array(
						'Project.id',
						// 'Agency.name',
						'Project.name',
						'Project.deadline_of_submission',
						'Project.pre_bidding_date',
						'Project.target_delivery_date',
						'ProjectStatus.name',
						'Project.name_of_company_won',
						'Project.lock_flag_name',
						'Project.status_name'
					),
					'data' => $project_list
				));
				
			}
			catch(NotFoundException $e) {
				$link_params = array_merge($this->request->params['pass'], $this->params['named'], array('?' => $this->request['url']));
				unset($link_params['page']);

				return $this->redirect($link_params);
			}
		}
		else {
			$project_list = $this->PpelcEntryItem->find('all', array( 
				'conditions' => $this->default_conditions,
				'order' => $default_order,
				'recursive' => -1
			));
				
			$project_list = reorder(array(
				'fields' => array(
					'Project.id',
					// 'Agency.name',
					'Project.name',
					'Project.deadline_of_submission',
					'Project.pre_bidding_date',
					'Project.target_delivery_date',
					'ProjectStatus.name',
					'Project.name_of_company_won',
					'Project.lock_flag_name',
					'Project.status_name'
				),
				'data' => $project_list
			));
			
		}
		
		return $project_list;
	}
	
	public function bidding_logout(){
		return $this->redirect($this->Auth->logout());
	}
	
	function beforeFilter(){
		parent::beforeFilter();
		$this->Auth->allow('bidding_logout');
	}
 
	public function isAuthorized($agency) {
		return $this->DaiSharedFunction->check_agency_privileges($agency);
	}
}
?>