<?php
App::uses('Component', 'Controller');

class DaiConditionBuilderComponent extends Component {
	
	public $controller;

	public function startup(Controller $controller) {
		$this->controller = $controller;
	}
	
    public function build_like($column_name, $form_name, $field_name, $params = '%LIKE%') {
		$new_condition = array();

		if(isset($this->controller->request->query[$field_name]) && trim($this->controller->request->query[$field_name]) != '') {
			$data = $this->controller->request->query[$field_name];
			$this->controller->request->data[$form_name][$field_name] = $data;
			
			if(is_array($column_name)) {
				$additional_condition = array();
				foreach($column_name as $column_value) {
					if($params == 'LIKE' || $params == '%LIKE%') {
						$additional_condition[$column_value . ' LIKE'] = '%' . $data . '%';
					}
					else if($params == 'LIKE%') {
						$additional_condition[$column_value . ' LIKE'] = $data . '%';
					}
					else if($params == '%LIKE') {
						$additional_condition[$column_value . ' LIKE'] = '%' . $data;
					}
				}
				
				$new_condition['OR'] = array_merge($new_condition, $additional_condition);
			}
			else {				
				if($params == 'LIKE' || $params == '%LIKE%') {
					$new_condition = array(
						$column_name . ' LIKE' => '%' . $data . '%'
					);
				}
				else if($params == 'LIKE%') {
					$new_condition = array(
						$column_name . ' LIKE' => $data . '%'
					);
				}
				else if($params == '%LIKE') {
					$new_condition = array(
						$column_name . ' LIKE' => '%' . $data
					);
				}
			}
		}
		
		return $new_condition;
    }
	
	public function build_status($column_name, $form_name, $field_name) {
		$new_condition = array();

		if(isset($this->controller->request->query[$field_name])) {
			$status = $this->controller->request->query[$field_name];
			$this->controller->request->data[$form_name][$field_name] = $status;
			
			if(trim($this->controller->request->query[$field_name]) != '') {
				$new_condition = array(
					$column_name => $status
				);
			}
		}
		else {
			$this->controller->request->data[$form_name][$field_name] = 1;
			
			$new_condition = array(
				$column_name => 1
			);
		}
		
		return $new_condition;
    }
	
	public function build_string_equivalent($column_name, $form_name, $field_name) {
		$new_condition = array();
		
		if(isset($this->controller->request->query[$field_name]) && trim($this->controller->request->query[$field_name]) != '') {
			$data = $this->controller->request->query[$field_name];
			$this->controller->request->data[$form_name][$field_name] = $data;

			$new_condition = array(
				$column_name => $data
			);
		}
		
		return $new_condition;
	}
	
	public function build_date_equivalent($column_name, $form_name, $field_name) {
		$new_condition = array();
		
		if(isset($this->controller->request->query[$field_name]) && !empty($this->controller->request->query[$field_name])) {
			$data = convert_datetime($this->controller->request->query[$field_name], 'Y-m-d');
			$this->controller->request->data[$form_name][$field_name] = convert_datetime($data, 'm/d/Y');

			$new_condition = array(
				"date(".$column_name.")" => $data
			);
		}
		
		return $new_condition;
	}
	
	public function build_date_comparison($column_name, $form_name, $field_name, $field_name_comparison_operator = null) {
		$new_condition = array();
		
		if(!isset($field_name_comparison_operator)) {
			$field_name_comparison_operator = $field_name . '_operator';
		}
		
		if((isset($this->controller->request->query[$field_name . '_value']) && trim($this->controller->request->query[$field_name . '_value']) != '')) {
			$field_name .= '_value';
		}
		
		if((isset($this->controller->request->query[$field_name . '_value']) && trim($this->controller->request->query[$field_name . '_value']) != '')) {
			$data = $this->controller->request->query[$field_name];
			$this->controller->request->data[$form_name][$field_name] = $data;
			
			$data_operator = $this->controller->request->query[$field_name_comparison_operator];
			$this->controller->request->data[$form_name][$field_name_comparison_operator] = $data_operator;
			
			if($data_operator == '=') {
				$data_operator = '';
			}
			
			$new_condition = array(
				trim('date(' . $column_name . ') ' . $data_operator) => $data
			);
		}
		else if(isset($this->controller->request->query[$field_name]) && trim($this->controller->request->query[$field_name]) != ''){
			$data = convert_datetime($this->controller->request->query[$field_name], 'Y-m-d');
			$this->controller->request->data[$form_name][$field_name] = convert_datetime($data, 'm/d/Y');
			
			$data_operator = $field_name_comparison_operator;
			$this->controller->request->data[$form_name][$field_name_comparison_operator] = $data_operator;
			
			if($data_operator == '=') {
				$data_operator = '';
			}
			
			$new_condition = array(
				trim('date(' . $column_name . ') ' . $data_operator) => $data
			);
		}
		
		return $new_condition;
	}
	
	public function build_numeric_comparison($column_name, $form_name, $field_name_comparison_operator, $field_name) {
		$new_condition = array();
		
		if((isset($this->controller->request->query[$field_name]) && $this->controller->request->query[$field_name] != '') && (isset($this->controller->request->query[$field_name_comparison_operator]) && $this->controller->request->query[$field_name_comparison_operator] != '')) {
			$data = $this->controller->request->query[$field_name];
			$this->controller->request->data[$form_name][$field_name] = $data;
			
			$data_operator = $this->controller->request->query[$field_name_comparison_operator];
			$this->controller->request->data[$form_name][$field_name_comparison_operator] = $data_operator;
			
			if($data_operator == '=') {
				$data_operator = '';
			}
			
			$new_condition = array(
				trim($column_name . ' ' . $data_operator) => $data
			);
		}
		
		return $new_condition;
	}
	
	public function build_date_range($column_name, $form_name, $field_name, $convert_to_month = false) {
		$new_condition = array();
		$hh_mm_dd = '';
		
		if($convert_to_month) {
			$hh_mm_dd = ' 23:59:59';
		}
		
		$from_field_name = $field_name . '_from';
		$to_field_name = $field_name . '_to';
		
		if(isset($this->controller->request->query[$from_field_name]) && trim($this->controller->request->query[$from_field_name]) != '') {
			$data = convert_datetime_from_format($this->controller->request->query[$from_field_name], 'm/d/Y');
			$this->controller->request->data[$form_name][$from_field_name] = $data->format('m/d/Y');
			
			$new_condition = array_merge(
				$new_condition,
				array(
					$column_name . ' >=' => $data->format('Y-m-d')
				)
			);
		}
		
		if(isset($this->controller->request->query[$to_field_name]) && trim($this->controller->request->query[$to_field_name]) != '') {
			$data = convert_datetime_from_format($this->controller->request->query[$to_field_name], 'm/d/Y');
			$this->controller->request->data[$form_name][$to_field_name] = $data->format('m/d/Y');
			
			$new_condition = array_merge(
				$new_condition,
				array(
					$column_name . ' <=' => $data->format('Y-m-d') . $hh_mm_dd
				)
			);
		}
		
		return $new_condition;
	}
	
	public function build_numeric_comparison2($column_name, $form_name, $field_name, $field_name_comparison_operator = null) {
		$new_condition = array();
		
		if(!isset($field_name_comparison_operator)) {
			$field_name_comparison_operator = $field_name . '_operator';
		}
		
		if((isset($this->controller->request->query[$field_name . '_value']) && trim($this->controller->request->query[$field_name . '_value']) != '')) {
			$field_name .= '_value';
		}
		
		if((isset($this->controller->request->query[$field_name . '_value']) && trim($this->controller->request->query[$field_name . '_value']) != '')) {
			$data = $this->controller->request->query[$field_name];
			$this->controller->request->data[$form_name][$field_name] = $data;
			
			$data_operator = $this->controller->request->query[$field_name_comparison_operator];
			$this->controller->request->data[$form_name][$field_name_comparison_operator] = $data_operator;
			
			if($data_operator == '=') {
				$data_operator = '';
			}
			
			$new_condition = array(
				trim($column_name . ' ' . $data_operator) => $data
			);
		}
		else if(isset($this->controller->request->query[$field_name]) && trim($this->controller->request->query[$field_name]) != ''){
			$data = $this->controller->request->query[$field_name];
			$this->controller->request->data[$form_name][$field_name] = $data;
			
			$data_operator = $this->controller->request->query[$field_name_comparison_operator];
			$this->controller->request->data[$form_name][$field_name_comparison_operator] = $data_operator;
			
			if($data_operator == '=') {
				$data_operator = '';
			}
			
			$new_condition = array(
				trim($column_name . ' ' . $data_operator) => $data
			);
		}
		
		return $new_condition;
	}
	
	public function build_text_comparison($column_name, $form_name, $field_name, $field_name_comparison_operator = null) {
		$new_condition = array();
		
		if(!isset($field_name_comparison_operator)) {
			$field_name_comparison_operator = $field_name . '_operator';
		}
		
		if((isset($this->controller->request->query[$field_name . '_value']) && trim($this->controller->request->query[$field_name . '_value']) != '')) {
			$field_name .= '_value'; 
			if (is_array($column_name)) {
				$additional_condition = array();
				foreach($column_name as $column_value) { 
					$additional_condition[] = $this->text_comparison_selector($column_value, $form_name, $field_name, $field_name_comparison_operator);
				} 
				$new_condition['OR'] = array_merge($new_condition, $additional_condition);
			}
			else {
				$new_condition = $this->text_comparison_selector($column_name, $form_name, $field_name, $field_name_comparison_operator);
			} 
		}
		else if(isset($this->controller->request->query[$field_name]) && trim($this->controller->request->query[$field_name]) != ''){ 
			if (is_array($column_name)) {
				$additional_condition = array();
				foreach($column_name as $column_value) { 
					$additional_condition[] = $this->text_comparison_selector($column_value, $form_name, $field_name, $field_name_comparison_operator);
				} 
				$new_condition['OR'] = array_merge($new_condition, $additional_condition);
			}
			else {
				$new_condition = $this->text_comparison_selector($column_name, $form_name, $field_name, $field_name_comparison_operator);
			}
		}
		else {
			$this->controller->request->data[$form_name][$field_name_comparison_operator] = 'has';
		}
		
		return $new_condition;
	}
	
	private function text_comparison_selector($column_name, $form_name, $field_name, $field_name_comparison_operator) {
		$new_condition = array();
		
		$data = $this->controller->request->query[$field_name];

		$this->controller->request->data[$form_name][$field_name] = $data;
		
		$data_operator = $this->controller->request->query[$field_name_comparison_operator];
		$this->controller->request->data[$form_name][$field_name_comparison_operator] = $data_operator;
		
		switch($data_operator) {
			case 'has':
			
				$new_condition = array(
					$column_name . ' LIKE' => '%' . $data . '%'
				);
				
			break;
			
			case 'starts_with':
			
				$new_condition = array(
					$column_name . ' LIKE' => $data . '%'
				);
				
			break;
			
			case 'ends_with':
			
				$new_condition = array(
					$column_name . ' LIKE' => '%' . $data
				);
				
			break;
			
			case 'exactly':
			
				$new_condition = array(
					$column_name => $data
				);
				
			break;
			
			default:
				$new_condition = array(
					$column_name . ' LIKE' => '%' . $data . '%'
				);
		}
		
		return $new_condition;
	}
}
?>