<?php
App::uses('Component', 'Controller');

class DaiSharedFunctionComponent extends Component {
	public $controller;
	
	public $components = array('Alert', 'DaiSecurity');
	public function initialize(Controller $controller) {
		$this->controller = $controller;
	}
	public function startup(Controller $controller) {
		$this->controller = $controller;
	}

	public function add_record(&$model, $return_address, $singular = true) {
		$model->attach_model();
		
		if($this->controller->request->is('put') || $this->controller->request->is('post')) {
			if($singular) {
				$alert_object = $this->controller->singular_alert_object;
			}
			else {
				$alert_object = $this->controller->plural_alert_object;
			}
			$data = $model->getDataSource();
			$data->begin();

			if($model->add_record($this->controller->request->data)) { 
				$data->commit();
				$this->controller->Alert->success_add($alert_object, $singular);
				return $this->controller->redirect($return_address);
			}
			else {
				$data->rollback(); 
				$this->Alert->failed_add($alert_object, $singular);				
			}
		}
		
		$this->controller->set('request_data', $this->controller->request->data);
	}
	
	public function add_record_with_no_layout_option(&$model, $return_address, $no_layout_address, $singular = true) {
		if(isset($this->controller->request->query['no-layout']) && $this->controller->request->query['no-layout'] == 1) {
			$this->controller->layout = 'inspinia_default';
			$return_address = $no_layout_address;
		}
		
		$this->add_record($model, $return_address, $singular);
	}
	
	public function get_list(&$model) {
		$this->controller->autoRender = false;
		$this->controller->layout = 'ajax';
		
		$records = $model->get_list();
		
		return json_encode($records);
	}
	
	public function edit_record(&$model, $return_address_on_success, $return_address_on_fail, $set_name, $id = null, $singular = true, $custom_find_function = 'find_by_id') {
		$model->attach_model();
		if($id) {
			$record = $model->$custom_find_function($id);
		
			if($record) {
				if($this->controller->request->is('put') || $this->controller->request->is('post')) {
					if($singular) {
						$alert_object = $this->controller->singular_alert_object;
					}
					else {
						$alert_object = $this->controller->plural_alert_object;
					}
				
					if($model->update_record($this->controller->request->data)) {
						$this->controller->Alert->success_update($alert_object, $singular);  
						return $this->controller->redirect($return_address_on_success);
					}
					else {
						$this->controller->Alert->failed_update($alert_object, $singular); 
						$this->controller->set($set_name, $record);					
					}
				}
				else {
					if($record[$model->name]['lock_flag']) {
						$this->controller->Alert->display('record_locked');
						return $this->controller->redirect($return_address_on_fail);
					}
					else {
						$this->controller->request->data = $record;
						$this->controller->set($set_name, $record);
					}
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				return $this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
		
		$this->controller->set('request_data', $this->controller->request->data);
	}
	
	public function edit_record_and_delete_dependents(&$model, $secondary_model_name, $foreign_key, $return_address_on_success, $return_address_on_fail, $set_name, $id = null, $singular = true, $custom_find_function = 'find_by_id') {
		$model->attach_model();
		if($id) {
			$record = $model->$custom_find_function($id);
		
			if($record) {
				if($this->controller->request->is('put') || $this->controller->request->is('post')) {
					$this->controller->loadModel($secondary_model_name);
					
					if($singular) {
						$alert_object = $this->controller->singular_alert_object;
					}
					else {
						$alert_object = $this->controller->plural_alert_object;
					}
					
					$secondary_data_source = $this->controller->$secondary_model_name->getDataSource();
					$secondary_data_source->begin();
					$secondary_record_ids = array();
					
					if (isset($this->controller->request->data[$this->controller->$secondary_model_name->name]) && count($this->controller->request->data[$this->controller->$secondary_model_name->name]) > 0) {
						foreach($this->controller->request->data[$this->controller->$secondary_model_name->name] as $secondary_record_item_key => $secondary_record_item_value) { 
							if (isset($secondary_record_item_value['id'])) {
								array_push($secondary_record_ids, $secondary_record_item_value['id']);
							}
						}
						
						
						$this->controller->$secondary_model_name->deleteAll(
							array(
								'NOT' => array(
									$this->controller->$secondary_model_name->name . '.id' => $secondary_record_ids
								),
								'AND' => array(
									$this->controller->$secondary_model_name->name . '.' . $foreign_key => $record[$model->name]['id']
								)
							), 
							false
						); 
					}
					else {
						$this->controller->$secondary_model_name->deleteAll(
							array( 
								'AND' => array(
									$this->controller->$secondary_model_name->name . '.' . $foreign_key => $record[$model->name]['id']
								)
							), 
							false
						); 
					}
				
					if($model->update_record($this->controller->request->data)) {
						$secondary_data_source->commit();
						$this->controller->Alert->success_update($alert_object, $singular);  
						return $this->controller->redirect($return_address_on_success);
					}
					else {
						$secondary_data_source->rollback();
						$secondary_data_source->commit();
						$this->controller->Alert->failed_update($alert_object, $singular); 
						$this->controller->set($set_name, $record);					
					}
				}
				else {
					if($record[$model->name]['lock_flag']) {
						$this->controller->Alert->display('record_locked');
						return $this->controller->redirect($return_address_on_fail);
					}
					else {
						$this->controller->request->data = $record;
						$this->controller->set($set_name, $record);
					}
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				return $this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
		
		$this->controller->set('request_data', $this->controller->request->data);
	}
	
	public function override_record(&$model, $return_address_on_success, $return_address_on_fail, $set_name, $id = null, $singular = true, $custom_find_function = 'find_by_id') {
		$model->attach_model();
		if($id) {
			$record = $model->$custom_find_function($id);
		
			if($record) {
				if($this->controller->request->is('put') || $this->controller->request->is('post')) {
					if($singular) {
						$alert_object = $this->controller->singular_alert_object;
					}
					else {
						$alert_object = $this->controller->plural_alert_object;
					}
				
					if($model->update_record($this->controller->request->data)) {
						$this->controller->Alert->success_update($alert_object, $singular);  
						return $this->controller->redirect($return_address_on_success);
					}
					else {
						$this->controller->request->data = $record;
						$this->controller->request->data = $this->controller->request->data;
						$this->controller->set($set_name, $record);						
						$this->controller->Alert->failed_update($alert_object, $singular); 
					}
				}
				else {
					$this->controller->request->data = $record;
					$this->controller->set($set_name, $record);
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				return $this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function edit_and_lock_record(&$model, $return_address_on_success, $return_address_on_fail, $set_name, $id = null, $singular = true, $custom_find_function = 'find_by_id') {
		if($id) {
			$is_already_used = $model->is_already_used($id, false, true);
			
			$model->attach_model();
			$record = $model->$custom_find_function($id);
		
			if($record) {
				if($is_already_used) {
					$this->controller->Alert->display('record_locked');
					return $this->controller->redirect($return_address_on_fail);
				}
				
				if($this->controller->request->is('put') || $this->controller->request->is('post')) {
					if($singular) {
						$alert_object = $this->controller->singular_alert_object;
					}
					else {
						$alert_object = $this->controller->plural_alert_object;
					}
					
					if($model->update_record($this->controller->request->data)) {
						$this->controller->Alert->success_update($alert_object, $singular);  
						return $this->controller->redirect($return_address_on_success);
					}
					else {
						$this->controller->Alert->failed_update($alert_object, $singular); 
						$this->controller->set($set_name, $record);						
					}
				}
				else {
					if($record[$model->name]['lock_flag']) {
						$this->controller->Alert->display('record_locked');
						return $this->controller->redirect($return_address_on_fail);
					}
					else {
						$this->controller->request->data = $record;
						$this->controller->set($set_name, $record);
					}
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				return $this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function view_record(&$model, $return_address, $set_name, $id = null, $custom_find_function = 'find_by_id') {
		$record = array();
		$model->attach_model();
		
		if($id) {
			$record = $model->$custom_find_function($id);
			
			if($record) {
				$this->controller->set($set_name, $record);
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address);
		}
		
		return $record;
	}
	
	public function view_and_lock_record(&$model, $return_address, $set_name, $id = null, $custom_find_function = 'find_by_id') {
		$record = array();
		
		if($id) {
			$model->is_already_used($id, false, true);
			
			$model->attach_model();
			
			$record = $model->$custom_find_function($id);
			
			if($record) {
				$this->controller->set($set_name, $record);
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address);
		}
		
		return $record;
	}
	
	public function enable_record(&$model, $return_address_on_success, $return_address_on_fail, $id = null, $singular = true) {
		$model->attach_model();
		
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) {
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}
				
				if($model->enable_record($id)) { 
					$this->controller->Alert->success_enabled($alert_object, $singular = true);  
					return $this->controller->redirect($return_address_on_success);
				}
				else {
					$this->controller->Alert->failed_enabled($alert_object, $singular = true);  
					return $this->controller->redirect($return_address_on_success);
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function disable_record(&$model, $return_address_on_success, $return_address_on_fail, $id = null, $singular = true) {
		$model->attach_model();
		
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) {
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}
				
				if($model->disable_record($id)) {
					$this->controller->Alert->success_disabled($alert_object, $singular); 
					return $this->controller->redirect($return_address_on_success);
				}
				else {
					$this->controller->Alert->failed_disabled($alert_object, $singular); 
					return $this->controller->redirect($return_address_on_success);
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function monitor_record(&$model, $return_address_on_success, $return_address_on_fail, $id = null, $singular = true) {
		$model->attach_model();
		
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) {
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}

				if($model->monitor_record($id)) { 
					$this->controller->Alert->success_monitored($alert_object, $singular = true);  
					return $this->controller->redirect($return_address_on_success);
				}
				else {
					$this->controller->Alert->failed_monitored($alert_object, $singular = true);  
					return $this->controller->redirect($return_address_on_success);
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function unmonitor_record(&$model, $return_address_on_success, $return_address_on_fail, $id = null, $singular = true) {
		$model->attach_model();
		
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) {
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}
				
				if($model->unmonitor_record($id)) {
					$this->controller->Alert->success_unmonitored($alert_object, $singular); 
					return $this->controller->redirect($return_address_on_success);
				}
				else {
					$this->controller->Alert->failed_unmonitored($alert_object, $singular); 
					return $this->controller->redirect($return_address_on_success);
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function include_in_alert_record(&$model, $return_address_on_success, $return_address_on_fail, $id = null, $singular = true) {
		$model->attach_model();
		
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) {
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}
				
				if($model->include_in_alert_record($id)) { 
					$this->controller->Alert->success_include_in_alert($alert_object, $singular = true);  
					return $this->controller->redirect($return_address_on_success);
				}
				else {
					$this->controller->Alert->failed_include_in_alert($alert_object, $singular = true);  
					return $this->controller->redirect($return_address_on_success);
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function uninclude_in_alert_record(&$model, $return_address_on_success, $return_address_on_fail, $id = null, $singular = true) {
		$model->attach_model();
		
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) {
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}
				
				if($model->uninclude_in_alert_record($id)) {
					$this->controller->Alert->success_uninclude_in_alert($alert_object, $singular); 
					return $this->controller->redirect($return_address_on_success);
				}
				else {
					$this->controller->Alert->failed_uninclude_in_alert($alert_object, $singular); 
					return $this->controller->redirect($return_address_on_success);
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function post_record(&$model, $return_address_on_success, $return_address_on_fail, $id = null, $singular = true) {
		$model->attach_model();
		
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) {
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}
				
				if($model->post_record($id)) { 
					$this->controller->Alert->success_post($alert_object, $singular = true);  
					return $this->controller->redirect($return_address_on_success);
				}
				else {
					$this->controller->Alert->failed_post($alert_object, $singular = true);  
					return $this->controller->redirect($return_address_on_success);
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address_on_fail);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address_on_fail);
		}
	}
	
	public function delete_record(&$model, $return_address, $id = null, $singular = true) {
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) { 
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}
				
				$model->attach_model();
				
				if($model->delete_record($id)) { 
					$this->controller->Alert->success_delete($alert_object, $singular); 
					$this->controller->redirect($return_address);
				}
				else {
					$this->controller->Alert->failed_delete($alert_object, $singular); 
					$this->controller->redirect($return_address);					
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address);
		}
	}
	
	public function deep_delete_record(&$model, $return_address, $id = null, $singular = true) {
		$model->attach_model();
		
		if($id) {
			$record = $model->find_by_id($id);
			
			if($record) { 
				if($singular) {
					$alert_object = $this->controller->singular_alert_object;
				}
				else {
					$alert_object = $this->controller->plural_alert_object;
				}
				
				if($model->deep_delete_record($id)) { 
					$this->controller->Alert->success_delete($alert_object, $singular); 
					$this->controller->redirect($return_address);
				}
				else {
					$this->controller->Alert->failed_delete($alert_object, $singular); 
					$this->controller->redirect($return_address);					
				}
			}
			else {
				$this->controller->Alert->display('no_records_found');
				$this->controller->redirect($return_address);
			}
		}
		else {
			$this->controller->Alert->display('no_records_found');
			$this->controller->redirect($return_address);
		}
	}
	
	public function get_records($model, $pagination_fields, $pagination_fields_reorder, $report_fields, $report_fields_reorder, $default_contain, $default_conditions, $default_order, $default_recursive, $is_paginated = true, $default_joins = array()) {
		$model->attach_model();
		
		if(isset($this->controller->request['url']['sort']) && $this->controller->request['url']['direction']) {
			$default_order = $this->controller->request['url']['sort'] . ' ' . $this->controller->request['url']['direction'];
		}
		
		if($is_paginated) {
			$this->controller->Paginator->settings = array(
				'fields' => $pagination_fields,
				'contain' => $default_contain,
				'conditions' => $default_conditions,
				'order' => $default_order,
				'limit' => Configure::read('PAGE_LIMIT'),
				'recursive' => $default_recursive,
				'joins' => $default_joins
			);
			
			try {
				$records = $this->controller->Paginator->paginate($model->name); 
				$records = reorder(array(
					'fields' => $pagination_fields_reorder,
					'data' => $records
				));
			}
			catch(NotFoundException $e) {
				$link_params = array_merge($this->controller->request->params['pass'], $this->controller->params['named'], array('?' => $this->controller->request['url']));
				unset($link_params['page']);

				return $this->controller->redirect($link_params);
			}
		}
		else {
			$records = $model->find('all', array(
				'fields' => $report_fields,
				'contain' => $default_contain,
				'conditions' => $default_conditions,
				'order' => $default_order,
				'recursive' => $default_recursive,
				'joins' => $default_joins
			));
			
			$records = reorder(array(
				'fields' => $report_fields_reorder,
				'data' => $records
			));
		}
		
		return $records;
	}
	
	public function get_list_from_model($model_name, $list_name, $custom_list_function = 'get_list',$conditions=array()) {
		$this->controller->loadModel($model_name);
		if($conditions){
			$records = $this->controller->$model_name->$custom_list_function($conditions);
		}else{
			$records = $this->controller->$model_name->$custom_list_function();
		}
		
		$this->controller->set($list_name, $records);
		return $records;
	}
	
	public function check_user_privileges($user) {
		if(isset($user['id'])) {
			return $this->DaiSecurity->check_user_privileges($user, $this->controller->params['controller'], $this->controller->params['action'], $this->controller->params['prefix']);
		}
		else {
			return false;
		}
	}
}
?>