<?php
App::uses('Component', 'Controller');

class DaiValidationBuilderComponent extends Component {
	
	public $controller;

	public function startup(Controller $controller) {
		$this->controller = $controller;
	}
	
	public function validation_builder($fields_for_validation = array()) {
		$validation_output = array();
		
		if(!empty($fields_for_validation)) {
			foreach($fields_for_validation as $field_key => $field_values) {
				$validation_output = array_merge($validation_output, $this->validation_matching($field_values));
			}
		// debug($validation_output);
		// exit();
		}
		
		return $validation_output;
	}
	
	public function validation_matching($params = array()) {
		
		$validation_holder = array();
		$table_name = Inflector::slug(strtolower($params['name']), '_');
		
		switch($params['type']) {
			case 'text':
				$length = 150;
				
				// if(isset($params['length']) && $params['length'] != 0) {
					// $length = $params['length'];
				// }
				
				$validation_holder[$table_name]['length validation']['rule'] = array('maxLength', $length);
				$validation_holder[$table_name]['length validation']['message'] = array(Inflector::humanize($params['name']) . ' must be no longer than '. $length .' characters.');
				
				if(isset($params['is_required']) && $params['is_required'] != 1) {
					$validation_holder[$table_name]['length validation']['allowEmpty'] = true;							
				}
				
			break;
			
			case 'list':
				
				$validation_holder[$table_name]['list validation']['rule'] = array('naturalNumber', true);
				$validation_holder[$table_name]['list validation']['message'] = array(Inflector::humanize($params['name']) . ' must be a valid item from the list');
				
				if(isset($params['is_required']) && $params['is_required'] != 1) {
					$validation_holder[$table_name]['list validation']['allowEmpty'] = true;							
				}
				
				if(isset($params['linked_model']) && !empty($params['linked_model'])) {
					$model_name = str_replace(' ', '', Inflector::humanize($params['linked_model']));
					$validation_holder['set_models'][$params['linked_model'] . '_list'] = ClassRegistry::init($model_name)->get_list();
				}
				
			break;
			
			case 'numeric':
				
				$validation_holder[$table_name]['numeric validation']['rule'] = array('numeric');
				$validation_holder[$table_name]['numeric validation']['message'] = array(Inflector::humanize($params['name']) . ' must be a valid numeric value');
				
				if(isset($params['is_required']) && $params['is_required'] != 1) {
					$validation_holder[$table_name]['numeric validation']['allowEmpty'] = true;							
				}
				
			break;
			
			case 'date':
				
				$validation_holder[$table_name]['date validation']['rule'] = array('date', 'mdy');
				$validation_holder[$table_name]['date validation']['message'] = array(Inflector::humanize($params['name']) . ' must be in mm/dd/yyyy format');
				
				if(isset($params['is_required']) && $params['is_required'] != 1) {
					$validation_holder[$table_name]['date validation']['allowEmpty'] = true;							
				}
				
			break;
			
			case 'checkbox':
				
				$validation_holder[$table_name]['checkbox validation']['rule'] = array('boolean');
				$validation_holder[$table_name]['checkbox validation']['message'] = array(Inflector::humanize($params['name']) . ' must be a boolean value');
				
				if(isset($params['is_required']) && $params['is_required'] != 1) {
					$validation_holder[$table_name]['checkbox validation']['allowEmpty'] = true;							
				}
				
			break;
		}
		// debug($validation_holder);
		// exit();
		return $validation_holder;
    }
	
	public function query_builder($params = array(), $return_string = true) {
		$query_array = array();
		
		$opening_statement = "(id mediumint(8) unsigned NOT NULL AUTO_INCREMENT" ;
		$closing_statement = "PRIMARY KEY (`id`), UNIQUE KEY `id_UNIQUE` (`id`))";
		
		$query_array = array_merge($query_array, array($opening_statement));
		
		foreach($params as $array_key => $array_value) {
			$merge_this_text = '';
			$column_name = Inflector::slug(strtolower($array_value['name']), '_');
	
			switch($array_value['type']) {
				case 'text':
					$length = 150;
					
					// if(isset($array_value['length']) && $array_value['length'] != 0) {
						// $length = $array_value['length'];
					// }						
					
					$merge_this_text = $column_name . ' VARCHAR(' . $length . ')';
					
				break;
				
				case 'list':
					
					$merge_this_text = $column_name . ' MEDIUMINT(8)';
					
				break;
				
				case 'numeric':
					
					$merge_this_text = $column_name . ' INT(8)';
					
				break;
				
				case 'date':
					
					$merge_this_text = $column_name . ' DATE';
					
				break;
				
				case 'checkbox':
					
					$merge_this_text = $column_name . ' TINYINT(1)';
					
				break;
			}
			
			if(isset($array_value['is_required']) && $array_value['is_required'] != 0) {
				$merge_this_text .= ' NOT NULL';					
			}
			
			$query_array = array_merge($query_array, array($merge_this_text));
		}
		
		$query_statement = array_merge($query_array, array($closing_statement));
		
		if($return_string) {			
			$query_statement = implode(',', $query_statement);
		}
		
		return $query_statement;
    }
	
	
}
?>