<?php
class ItemsController extends AppController {
	public $components = array('DaiSharedFunction');
  
	public $pagination_fields = array(
		'Item.id',
		'Item.brand',
		'Item.description',
		'Item.unit_of_measurement',
		'Item.lock_flag',
		'Item.status'
	);
	
	public $pagination_fields_reorder = array(
		'Item.id',
		'Item.brand',
		'Item.description',
		'Item.unit_of_measurement',
		'Item.lock_flag_name',
		'Item.status_name'
	);
			
	public $report_fields = array(
		'Item.id',
		'Item.brand',
		'Item.description',
		'Item.unit_of_measurement',
		'Item.lock_flag',
		'Item.status'
	);
	
	public $report_fields_reorder = array(
		'Item.brand',
		'Item.description',
		'Item.unit_of_measurement',
		'Item.lock_flag_name',
		'Item.status_name'
	);
	
	public $default_contain = array(
		
	);
	
	public $default_order = array('Item.brand' => 'ASC');
	
	public $default_conditions = array();
	
	public $default_recursive = -1;
	
	public $default_header_data = array(
		'Brand',
		'Description',
		'Unit Of Measurement',
		'Locked',
		'Status'
	);
	
	public $return_address = array('controller' => 'items', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_success = array('controller' => 'items', 'action' => 'view_item', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_add_no_layout = array('controller' => 'items', 'action' => 'add_item', 'prefix' => 'bidding', 'bidding' => true, '?' => array('no-layout' => 1));
	
	public $search_form_name = 'SearchItem';
	
	public $singular_alert_object = 'Item';
	
	public $plural_alert_object = 'Items';
	
	public $singular_set_name = 'item';
	
	public $plural_set_name = 'items';
	
	public function bidding_index() { 
		$records = $this->get_items();
	 
		if(!$records) {
			$this->Alert->display('no_records_found');
		}
		
		$this->set($this->plural_set_name, $records);
	}
	
	public function bidding_export_csv_item() { 
		
		$records = $this->get_items(
			array(),
			false
		);
		
		$this->Export->export_csv(array(
			'title' => 'Item List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_excel_item() { 
		
		$records = $this->get_items(
			array(),
			false
		);
		
		$this->Export->export_excel(array(
			'title' => 'Item List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_pdf_item() { 
		$records = $this->get_items(
			array(),
			false
		);
		
		$this->Export->export_pdf(array(
			'title' => 'Item List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}

	public function bidding_add_item() { 
		$this->DaiSharedFunction->add_record_with_no_layout_option(
			$this->Item, 
			$this->return_address,
			$this->return_address_on_add_no_layout
		); 
		
	}
	
	public function bidding_get_list() {
		return $this->DaiSharedFunction->get_list(
			$this->Item
		);
	}
	
	public function bidding_edit_item($id = null) { 
		$this->DaiSharedFunction->edit_and_lock_record(
			$this->Item, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_override_password($id = null) { 
		$this->DaiSharedFunction->override_record(
			$this->Item, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_change_password(){ 
		$this->Item->attach_model();
		
		if($this->request->is('put') || $this->request->is('post')) {
			if($this->Item->change_password($this->request->data)) {
				$this->Alert->display('success_change_password');
				return $this->redirect(array('controller' => 'home', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
			else {
				$this->Alert->display('failed_change_password');
			}
		}
	}
	
	public function bidding_view_item($id = null) { 
		$this->DaiSharedFunction->view_and_lock_record(
			$this->Item, 
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_enable_item($id = null) {  
		$this->DaiSharedFunction->enable_record(
			$this->Item, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_disable_item($id = null) { 
		
		$this->DaiSharedFunction->disable_record(
			$this->Item, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_delete_item($id = null) { 
		$this->DaiSharedFunction->delete_record(
			$this->Item, 
			$this->return_address,
			$id
		);
	}
	
	private function get_items($submitted_conditions = array(), $is_paginated = true) {
		$search_form_name = $this->search_form_name;
		
		$this->default_conditions = array();
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Item.brand', $search_form_name, 'brand')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Item.description', $search_form_name, 'description')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Item.unit_of_measurement', $search_form_name, 'unit_of_measurement')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Item.lock_flag', $search_form_name, 'locked'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_status('Item.status', $search_form_name, 'status'));
 
		return $this->DaiSharedFunction->get_records(
			$this->Item, 
			$this->pagination_fields,
			$this->pagination_fields_reorder,
			$this->report_fields,
			$this->report_fields_reorder,
			$this->default_contain,
			$this->default_conditions,
			$this->default_order,
			$this->default_recursive,
			$is_paginated
		); 
	}
	
	public function bidding_get_items(){
		$this->loadModel('Item'); 
		$this->layout = 'ajax';
		$search_keyword = '';
		$selected_item_id = '';  
		
		if (isset($this->request->data['search_keyword'])) {
			$search_keyword = $this->request->data['search_keyword'];
		}
		
		$default_conditions = array(
			'Item.status' => 1,
			'OR' => array(
				'Item.brand LIKE \'%'. $search_keyword . '%\'',
				'Item.description LIKE \'%'. $search_keyword . '%\'',
			)
		);

		 
 		
		
		if (isset($this->request->data['selected_item_ids']) && trim($this->request->data['selected_item_ids']) !== '') {
			$selected_item_id = $this->request->data['selected_item_ids'];
			$selected_item_id = explode('|', $selected_item_id);
			$default_conditions = array_merge($default_conditions, array(
				'not'=>array(
					'Item.id'=> $selected_item_id
				)
			));
		}
		
		$items = $this->Item->find('all', array(
			'fields' => array(
				'Item.id',
				'Item.brand',
				'Item.description',
				'Item.unit_of_measurement',
			),
			'conditions' => $default_conditions,
			'order' => 'Item.brand ASC',
			'recursive' => -1,
			'limit' => Configure::read('PAGE_LIMIT')
		)); 
		
		$this->set('items', $items);
		
	}
	
	public function bidding_logout(){
		return $this->redirect($this->Auth->logout());
	}
	
	function beforeFilter(){
		parent::beforeFilter();
		$this->Auth->allow('bidding_logout');
	}
 
	public function isAuthorized($item) {
		return $this->DaiSharedFunction->check_item_privileges($item);
	}
}
?>