<?php
class ProjectStatusesController extends AppController {
	public $components = array('DaiSharedFunction');
  
	public $pagination_fields = array(
		'ProjectStatus.id',
		'ProjectStatus.name',
		'ProjectStatus.lock_flag',
		'ProjectStatus.status'
	);
	
	public $pagination_fields_reorder = array(
		'ProjectStatus.id',
		'ProjectStatus.name',
		'ProjectStatus.lock_flag_name',
		'ProjectStatus.status_name'
	);
			
	public $report_fields = array(
		'ProjectStatus.id',
		'ProjectStatus.name',
		'ProjectStatus.lock_flag',
		'ProjectStatus.status'
	);
	
	public $report_fields_reorder = array(
		'ProjectStatus.name',
		'ProjectStatus.lock_flag_name',
		'ProjectStatus.status_name'
	);
	
	public $default_contain = array(
		
	);
	
	public $default_order = array('ProjectStatus.name' => 'ASC');
	
	public $default_conditions = array();
	
	public $default_recursive = -1;
	
	public $default_header_data = array(
		'Brand',
		'Locked',
		'Status'
	);
	
	public $return_address = array('controller' => 'project_statuses', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_success = array('controller' => 'project_statuses', 'action' => 'view_project_status', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_add_no_layout = array('controller' => 'project_statuses', 'action' => 'add_project_status', 'prefix' => 'bidding', 'bidding' => true, '?' => array('no-layout' => 1));
	
	public $search_form_name = 'SearchProjectStatus';
	
	public $singular_alert_object = 'ProjectStatus';
	
	public $plural_alert_object = 'ProjectStatuses';
	
	public $singular_set_name = 'project_status';
	
	public $plural_set_name = 'project_statuses';
	
	public function bidding_index() { 
		$records = $this->get_project_statuses();
	 
		if(!$records) {
			$this->Alert->display('no_records_found');
		}
		
		$this->set($this->plural_set_name, $records);
	}
	
	public function bidding_export_csv_project_status() { 
		
		$records = $this->get_project_statuses(
			array(),
			false
		);
		
		$this->Export->export_csv(array(
			'title' => 'ProjectStatus List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_excel_project_status() { 
		
		$records = $this->get_project_statuses(
			array(),
			false
		);
		
		$this->Export->export_excel(array(
			'title' => 'ProjectStatus List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_pdf_project_status() { 
		$records = $this->get_project_statuses(
			array(),
			false
		);
		
		$this->Export->export_pdf(array(
			'title' => 'ProjectStatus List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}

	public function bidding_add_project_status() { 
		$this->DaiSharedFunction->add_record_with_no_layout_option(
			$this->ProjectStatus, 
			$this->return_address,
			$this->return_address_on_add_no_layout
		); 
		
	}
	
	public function bidding_get_list() {
		return $this->DaiSharedFunction->get_list(
			$this->ProjectStatus
		);
	}
	
	public function bidding_edit_project_status($id = null) { 
		$this->DaiSharedFunction->edit_and_lock_record(
			$this->ProjectStatus, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_override_password($id = null) { 
		$this->DaiSharedFunction->override_record(
			$this->ProjectStatus, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_change_password(){ 
		$this->ProjectStatus->attach_model();
		
		if($this->request->is('put') || $this->request->is('post')) {
			if($this->ProjectStatus->change_password($this->request->data)) {
				$this->Alert->display('success_change_password');
				return $this->redirect(array('controller' => 'home', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
			else {
				$this->Alert->display('failed_change_password');
			}
		}
	}
	
	public function bidding_view_project_status($id = null) { 
		$this->DaiSharedFunction->view_and_lock_record(
			$this->ProjectStatus, 
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_enable_project_status($id = null) {  
		$this->DaiSharedFunction->enable_record(
			$this->ProjectStatus, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_disable_project_status($id = null) { 
		
		$this->DaiSharedFunction->disable_record(
			$this->ProjectStatus, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_delete_project_status($id = null) { 
		$this->DaiSharedFunction->delete_record(
			$this->ProjectStatus, 
			$this->return_address,
			$id
		);
	}
	
	private function get_project_statuses($submitted_conditions = array(), $is_paginated = true) {
		$search_form_name = $this->search_form_name;
		
		$this->default_conditions = array();
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('ProjectStatus.name', $search_form_name, 'name'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('ProjectStatus.lock_flag', $search_form_name, 'locked'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_status('ProjectStatus.status', $search_form_name, 'status'));
 
		return $this->DaiSharedFunction->get_records(
			$this->ProjectStatus, 
			$this->pagination_fields,
			$this->pagination_fields_reorder,
			$this->report_fields,
			$this->report_fields_reorder,
			$this->default_contain,
			$this->default_conditions,
			$this->default_order,
			$this->default_recursive,
			$is_paginated
		); 
	}
	
	public function bidding_logout(){
		return $this->redirect($this->Auth->logout());
	}
	
	function beforeFilter(){
		parent::beforeFilter();
		$this->Auth->allow('bidding_logout');
	}
 
	public function isAuthorized($project_status) {
		return $this->DaiSharedFunction->check_project_status_privileges($project_status);
	}
}
?>