<?php
class ProjectsController extends AppController {
	public $components = array('DaiSharedFunction');
  
	public $pagination_fields = array(
		'Project.id',
		'Agency.name',
		'Project.name',
		'Project.abc_amount',
		'Project.dai_amount_offered',
		'Project.date_downloaded',
		'Project.deadline_of_submission',
		'Project.delivery_period',
		'Project.requirements',
		'Project.pre_bidding_date',
		'Project.target_delivery_date',
		'Project.delivered_on',
		'ProjectStatus.name',
		'Project.name_of_company_won',
		'Project.winning_proposal_amount',
		'Project.remarks',
		'Project.lock_flag',
		'Project.status'
	);
	
	public $pagination_fields_reorder = array(
		'Project.id',
		'Agency.name',
		'Project.name',
		'Project.abc_amount_currency',
		'Project.dai_amount_offered_currency',
		'Project.date_downloaded',
		'Project.deadline_of_submission',
		'Project.delivery_period',
		'Project.requirements',
		'Project.pre_bidding_date',
		'Project.target_delivery_date',
		'Project.delivered_on',
		'ProjectStatus.name',
		'Project.name_of_company_won',
		'Project.winning_proposal_amount_currency',
		'Project.remarks',
		'Project.lock_flag_name',
		'Project.status_name'
	);
			
	public $report_fields = array(
		'Project.id',
		'Agency.name',
		'Project.name',
		'Project.abc_amount',
		'Project.dai_amount_offered',
		'Project.date_downloaded',
		'Project.deadline_of_submission',
		'Project.delivery_period',
		'Project.requirements',
		'Project.pre_bidding_date',
		'Project.target_delivery_date',
		'Project.delivered_on',
		'ProjectStatus.name',
		'Project.name_of_company_won',
		'Project.winning_proposal_amount',
		'Project.remarks',
		'Project.lock_flag',
		'Project.status'
	);
	
	public $report_fields_reorder = array(
		'Agency.name',
		'Project.name',
		'Project.abc_amount',
		'Project.dai_amount_offered',
		'Project.date_downloaded',
		'Project.deadline_of_submission',
		'Project.delivery_period',
		'Project.requirements',
		'Project.pre_bidding_date',
		'Project.target_delivery_date',
		'Project.delivered_on',
		'ProjectStatus.name',
		'Project.name_of_company_won',
		'Project.winning_proposal_amount',
		'Project.remarks',
		'Project.lock_flag_name',
		'Project.status_name'
	);
	
	public $default_contain = array(
		'Agency',
		'ProjectStatus'
	);
	
	public $default_order = array('Project.name' => 'ASC');
	
	public $default_conditions = array();
	
	public $default_recursive = -1;
	
	public $default_header_data = array(
		'Agency Name',
		'Project Name',
		'ABC Amount',
		'DAI Amount Offered',
		'Date Downloaded',
		'Deadline of Submission',
		'Delivery Period',
		'Requirements',
		'Pre Bidding Date',
		'Target Delivery Date',
		'Delivered On',
		'Project Status',
		'Name of Company Won',
		'Winning Proposal Amount',
		'Remarks',
		'Locked',
		'Status'
	);
	
	public $return_address = array('controller' => 'projects', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_success = array('controller' => 'projects', 'action' => 'view_project', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_add_no_layout = array('controller' => 'projects', 'action' => 'add_project', 'prefix' => 'bidding', 'bidding' => true, '?' => array('no-layout' => 1));
	
	public $search_form_name = 'SearchProject';
	
	public $singular_alert_object = 'Project';
	
	public $plural_alert_object = 'Projects';
	
	public $singular_set_name = 'project';
	
	public $plural_set_name = 'projects';
	
	public function bidding_index() { 
		$this->loadModel('ProjectStatus');
		$project_status_list = $this->ProjectStatus->get_list();
		$this->set('project_status_list', $project_status_list);
		
		$records = $this->get_projects();
		
		if(!$records) {
			$this->Alert->display('no_records_found');
		}
		
		$this->set($this->plural_set_name, $records);
	}
	
	public function bidding_export_csv_project() { 
		
		$records = $this->get_projects(
			array(),
			false
		);
		
		$this->Export->export_csv(array(
			'title' => 'Project List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_excel_project() { 
		
		$records = $this->get_projects(
			array(),
			false
		);
		
		$this->Export->export_excel(array(
			'title' => 'Project List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_pdf_project() { 
		$records = $this->get_projects(
			array(),
			false
		);
		
		$this->Export->export_pdf(array(
			'title' => 'Project List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}

	public function bidding_add_project() { 
		// $this->DaiSharedFunction->add_record_with_no_layout_option(
			// $this->Project, 0
			// $this->return_address,
			// $this->return_address_on_add_no_layout
		// ); 
			
		$this->DaiSharedFunction->get_list_from_model('Agency', 'agency_list');
		// $this->DaiSharedFunction->get_list_from_model('ProjectStatus', 'project_status_list');
		$this->loadModel('ProjectStatus');
		$project_status_list = $this->ProjectStatus->get_list();
		$this->set('project_status_list', $project_status_list);
		
		if($this->request->is('put') || $this->request->is('post')) {
			$data = array();
			unset($this->request->data['search_keyword']);
			
			$data = $this->request->data;
			
			$data['ProjectHistory'][0]['posted_by'] = $_SESSION['Auth']['User']['id'];
			$data['ProjectHistory'][0]['date_and_time'] = date('Y-m-d H:i:s');
			$data['ProjectHistory'][0]['description'] = 'Project has been added. (Status: ' . $project_status_list[$this->request->data['Project']['project_status_id']] . ')'; 
			// debug($this->request->data);
			// debug($data);
			// debug($_SESSION);
			// exit();

			$item_history_content = '';
			foreach($data['Proposal'] as $item_content_key => $item_content_value){
				// debug($item_content_value);
				$item_history_content .=  '<tr data-item-id="'. $item_content_value['item_id'] .'">
					<td>' . $item_content_value['brand'] . '</td>
					<td>' . $item_content_value['description'] . '</td>
					<td>' . $item_content_value['quantity'] . '</td>
					<td>' . $item_content_value['unit_of_measurement'] . '</td>
					<td>' . $item_content_value['unit_price'] . '</td>
					<td>' . $item_content_value['total'] . '</td>
				</tr>';
			}
			// exit();
			$item_history_description = '<div class="table-responsive">
				<table class="table table-bordered table-hover project-history-item-table">
					<thead>
						<tr>
							<th class="table-header-links col-md-1">Brand</th>
							<th class="table-header-links col-md-3">Description</th>
							<th class="table-header-links col-md-1">Quantity</th>
							<th class="table-header-links col-md-1">Unit</th>
							<th class="table-header-links col-md-1">Unit Price</th>
							<th class="table-header-links col-md-1">Total</th>
						</tr>
					</thead>
					<tbody>
						' . $item_history_content . '
					</tbody>
				</table>
			</div>';
			// debug($item_history_description);
			// exit();

			$data['ProjectHistory'][1]['posted_by'] = $_SESSION['Auth']['User']['id'];
			$data['ProjectHistory'][1]['date_and_time'] = date('Y-m-d H:i:s');
			$data['ProjectHistory'][1]['item_history_description'] = $item_history_description;
			
			foreach($data['Proposal'] as $table_item_key => $table_item_value){
				// debug($table_item_value);
				$data['Proposal'][$table_item_key]['unit_price'] = number_format($table_item_value['unit_price'], 2, '.','');
			}
			// exit();
			
			$json_item_table = json_encode($data['Proposal']);
			$data['Project']['item_table_json'] = $json_item_table;
			// debug($data);
			// debug($json_item_table);
			// exit();
			if($this->Project->saveAssociated($data, array('deep' => true))){
				$this->Session->setFlash('Project has been successfully added.' , 'alert.success-full');
				return $this->redirect(array('controller' => 'projects', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
			else{
				$this->Session->setFlash('Failed to approved. Please try again', 'alert.success-full');
				return $this->redirect(array('controller' => 'projects', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
		}
	}
	
	public function bidding_get_list() {
		return $this->DaiSharedFunction->get_list(
			$this->Project
		);
	}
	
	public function bidding_edit_project($id = null) { 
		$this->DaiSharedFunction->get_list_from_model('Agency', 'agency_list');
		// $this->DaiSharedFunction->get_list_from_model('ProjectStatus', 'project_status_list');
		$this->loadModel('ProjectStatus');
		$this->loadModel('Proposal');
		$this->loadModel('Attachment');
		$project_status_list = $this->ProjectStatus->get_list();
		$this->set('project_status_list', $project_status_list);
		
		$projects = $this->Project->find('first', array(
			'contain' => array(
				'Proposal' => array(
					'Item'
				),
				'Attachment'
			),
			'conditions' => array(
				'Project.id' => $id
			),
		));
		
		// debug($projects);
		// exit();
		if($this->request->is('put') || $this->request->is('post')) {
			$data = array();
			$project_data = $this->Project->getDataSource();
			$project_data->begin();
			unset($this->request->data['search_keyword']);

			$proposal_item_id = array();
			if (isset($this->request->data['Proposal']) && count($this->request->data['Proposal']) > 0) {
				foreach($this->request->data['Proposal'] as $proposal_item_key => $proposal_item_value) {
					if(isset($proposal_item_value['id']) && !empty($proposal_item_value['id'])){
						array_push($proposal_item_id, $proposal_item_value['id']);
					}
				}
				// exit();
				$this->Proposal->deleteAll(
					array(
						'NOT' => array(
							'Proposal.id' => $proposal_item_id
						),
						'AND' => array(
							'Proposal.project_id' => $this->request->data['Project']['id']
						)
					), 
					false
				); 
			}	
			$attachment_item_id = array();
			if (isset($this->request->data['Attachment']) && count($this->request->data['Attachment']) > 0) {
				foreach($this->request->data['Attachment'] as $attachment_item_key => $attachment_item_value) {
					// debug($attachment_item_value);
					if(isset($attachment_item_value['id']) && !empty($attachment_item_value['id'])){						
						array_push($attachment_item_id, $attachment_item_value['id']);
					}
				}
				// exit();
				$this->Attachment->deleteAll(
					array(
						'NOT' => array(
							'Attachment.id' => $attachment_item_id
						),
						'AND' => array(
							'Attachment.project_id' => $this->request->data['Project']['id']
						)
					), 
					false
				); 
			}	
			$data = $this->request->data;
			$project_difference = array_diff_assoc($data['Project'],$projects['Project']);
			$description = array();
			if(!empty($project_difference)){
				foreach($project_difference as $difference_key => $difference_value){
					if($difference_key == 'won_by_dai'){
						if ($projects['Project'][$difference_key] == 0){
							$projects['Project'][$difference_key] = "No";
						}
						else{
							$projects['Project'][$difference_key] = "Yes";
						}
						if($difference_value == 0){
							$difference_value = "No";
						}
						else{
							$difference_value = "Yes";
						}
					}
					$humanize_difference_key = Inflector::humanize($difference_key);
					$description[$difference_key] = $humanize_difference_key . ' has been changed from ' . $projects['Project'][$difference_key] . ' to ' . $difference_value;
					$description = array_merge($description,$description);
				}
				$description = implode('<hr>',$description);
				// debug($description);
				// exit();
				$data['ProjectHistory'][0]['posted_by'] = $_SESSION['Auth']['User']['id'];
				$data['ProjectHistory'][0]['date_and_time'] = date('Y-m-d H:i:s');
				$data['ProjectHistory'][0]['description'] = $description;
			}
			// debug($projects['Project']);
			
			foreach($data['Proposal'] as $table_item_key => $table_item_value){
				// debug($table_item_value);
				$data['Proposal'][$table_item_key]['unit_price'] = number_format($table_item_value['unit_price'], 2, '.','');
			}
			// exit();
			$json_item_table = json_encode($data['Proposal']);
			// debug($data);
			// debug($json_item_table);
			// debug($projects['Project']['item_table_json']);
			// exit();
			// $data['Project']['item_table_json'] = $json_item_table;
			if($projects['Project']['item_table_json'] != $json_item_table){
				$item_history_content = '';
				foreach($data['Proposal'] as $item_content_key => $item_content_value){
					// debug($item_content_value);
					$item_history_content .=  '<tr data-item-id="'. $item_content_value['item_id'] .'">
						<td>' . $item_content_value['brand'] . '</td>
						<td>' . $item_content_value['description'] . '</td>
						<td>' . $item_content_value['quantity'] . '</td>
						<td>' . $item_content_value['unit_of_measurement'] . '</td>
						<td>' . $item_content_value['unit_price'] . '</td>
						<td>' . $item_content_value['total'] . '</td>
					</tr>';
				}
				// exit();
				$item_history_description = '<div class="table-responsive">
					<table class="table table-bordered table-hover project-history-item-table">
						<thead>
							<tr>
								<th class="table-header-links col-md-1">Brand</th>
								<th class="table-header-links col-md-3">Description</th>
								<th class="table-header-links col-md-1">Quantity</th>
								<th class="table-header-links col-md-1">Unit</th>
								<th class="table-header-links col-md-1">Unit Price</th>
								<th class="table-header-links col-md-1">Total</th>
							</tr>
						</thead>
						<tbody>
							' . $item_history_content . '
						</tbody>
					</table>
				</div>';
				// debug($item_history_description);
				// exit();

				$data['ProjectHistory'][1]['posted_by'] = $_SESSION['Auth']['User']['id'];
				$data['ProjectHistory'][1]['date_and_time'] = date('Y-m-d H:i:s');
				$data['ProjectHistory'][1]['item_history_description'] = $item_history_description; 
			}
			
			

			
			// debug($data);
			// exit();
			if($this->Project->saveAssociated($data, array('deep' => true))){
				$project_data->commit();  
				$this->Session->setFlash('Project has been successfully added.' , 'alert.success-full');
				return $this->redirect(array('controller' => 'projects', 'action' => 'index', 'prefix' => 'dms', 'dms' => true));
			}
			else{
				$project_data->rollback();
				$this->Session->setFlash('Failed to approved. Please try again', 'alert.success-full');
				return $this->redirect(array('controller' => 'projects', 'action' => 'index', 'prefix' => 'dms', 'dms' => true));
			}
		}
		else{
			$this->request->data = $projects;
		}
	}
	
	public function bidding_override_password($id = null) { 
		$this->DaiSharedFunction->override_record(
			$this->Project, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_change_password(){ 
		$this->Project->attach_model();
		
		if($this->request->is('put') || $this->request->is('post')) {
			if($this->Project->change_password($this->request->data)) {
				$this->Alert->display('success_change_password');
				return $this->redirect(array('controller' => 'home', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
			else {
				$this->Alert->display('failed_change_password');
			}
		}
	}
	
	public function bidding_view_project($id = null) { 
		$project = $this->Project->find('first', array(
			'contain' => array(
				'Agency',
				'ProjectStatus',
				'Proposal' => array(
					'Item'
				),
				'Attachment'
			),
			'conditions' => array(
				'Project.id' => $id
			),
		));
		// debug($project);
		// exit();
		$href_link = $this->base;	
		foreach($project['Attachment'] as $project_attachment_item => $project_attachment_value){
			// debug($project_attachment_value);
			if($project_attachment_value['attachment'] == null){
				$attachment_link = '<i class="fa fa-file-o sm-icon" aria-hidden="true">' . ' Preview/Download' . '</i>';
			}
			else{
				$attachment_link ='<div class="text-center"><a href="'. $href_link .'/'.$project_attachment_value['attachment'].'" class="blue-link" target="_blank"><i class="fa fa-file-text-o sm-icon" aria-hidden="true">' . ' Preview/Download' . '</i></a></div>';
			}
			
			$project['Attachment'][$project_attachment_item]['attachment'] =  $attachment_link;
		}
		// debug($_SESSION);
		// exit();
		
		if ($project) {
			$project_history_logs = $this->get_project_history(
				array('ProjectHistory.project_id' => $project['Project']['id'])
			);
			if ($this->request->is('post') || $this->request->is('put')) {
				$this->request->data['ProjectHistory']['project_id'] = $id;
				$this->request->data['ProjectHistory']['posted_by'] = $_SESSION['Auth']['User']['id'];
				$this->request->data['ProjectHistory']['date_and_time'] = date('Y-m-d H:i:s');
				// debug($this->request->data);
				// exit();
				if ($this->ProjectHistory->save($this->request->data)){
					$this->Alert->success_add('Project Log');
					return $this->redirect(array('controller' => 'projects', 'action' => 'view_project', $id,'prefix' => 'bidding', 'bidding' => true));
				}
				else{
					
					$this->Alert->failed_add('Project Log');
					return $this->redirect(array('controller' => 'projects', 'action' => 'view_project', $id,'prefix' => 'bidding', 'bidding' => true));
				}
				
			}
		}
		else {
			$this->Alert->display('no_records_found');
			return $this->redirect($this->return_address);
		}
		
		$this->loadModel('User');
		$user_list = $this->User->get_name_of_user();
		$this->set('user_list',$user_list);
		$this->set('project_history_logs',$project_history_logs);
		$this->set('project',$project);
		
	}
	
	public function bidding_enable_project($id = null) {  
		$this->DaiSharedFunction->enable_record(
			$this->Project, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_disable_project($id = null) { 
		
		$this->DaiSharedFunction->disable_record(
			$this->Project, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_delete_project($id = null) { 
		$this->DaiSharedFunction->delete_record(
			$this->Project, 
			$this->return_address,
			$id
		);
	}
	
	private function get_projects($submitted_conditions = array(), $is_paginated = true) {
		$search_form_name = $this->search_form_name;
		
		$this->default_conditions = array();
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.name', $search_form_name, 'agency_name')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Project.name', $search_form_name, 'project_name'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_numeric_comparison2('Project.abc_amount', $search_form_name, 'abc_amount'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_numeric_comparison2('Project.dai_amount_offered', $search_form_name, 'dai_amount_offered'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_range('Project.date_downloaded', $search_form_name, 'date_downloaded'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_range('Project.deadline_of_submission', $search_form_name, 'deadline_of_submission'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_range('Project.delivery_period', $search_form_name, 'delivery_period'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Project.requirements', $search_form_name, 'requirements'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_range('Project.pre_bidding_date', $search_form_name, 'pre_bidding_date'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_equivalent('Project.target_delivery_date', $search_form_name, 'target_delivery_date'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_equivalent('Project.delivered_on', $search_form_name, 'delivered_on'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Project.project_status_id', $search_form_name, 'project_status'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Project.name_of_company_won', $search_form_name, 'name_of_company_won'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_numeric_comparison2('Project.winning_proposal_amount', $search_form_name, 'winning_proposal_amount'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Project.remarks', $search_form_name, 'remarks'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Project.lock_flag', $search_form_name, 'locked'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_status('Project.status', $search_form_name, 'status'));
		
		return $this->DaiSharedFunction->get_records(
			$this->Project, 
			$this->pagination_fields,
			$this->pagination_fields_reorder,
			$this->report_fields,
			$this->report_fields_reorder,
			$this->default_contain,
			$this->default_conditions,
			$this->default_order,
			$this->default_recursive,
			$is_paginated
		); 
	}
	
	
	public function bidding_item_report_index(){
		$this->loadModel('ProjectStatus');
		$project_status_list = $this->ProjectStatus->get_list();
		$this->set('project_status_list', $project_status_list);
		
		$this->search_item_report();
	}
	
	public function bidding_export_excel_item_report() {
		$this->search_item_report(
			array(),
			'Basic Reports',
			'excel'
		);
		
	}
	
	private function search_item_report($submitted_conditions = array(), $report_type = '', $report_format = ''){	
		$search_form_name = $this->search_form_name;
		// debug($this->request->data);
		// exit();
		$this->default_conditions = array();
		// $this->request->query['agency_name_value']
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Item.brand', $search_form_name, 'brand')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Item.description', $search_form_name, 'description'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Item.unit_of_measurement', $search_form_name, 'unit'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_numeric_comparison2('Project.abc_amount', $search_form_name, 'abc_amount'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_numeric_comparison2('Project.dai_amount_offered', $search_form_name, 'dai_amount_offered'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_numeric_comparison2('Proposal.quantity', $search_form_name, 'quantity'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_numeric_comparison2('Proposal.unit_price', $search_form_name, 'unit_price'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_numeric_comparison2('Proposal.total', $search_form_name, 'total'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Agency.name', $search_form_name, 'agency_name'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Project.name', $search_form_name, 'project_name'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Project.project_status_id', $search_form_name, 'project_status'));
		// debug($this->default_conditions);
		// exit();
		
		if(isset($report_type) && $report_type == 'Basic Reports') {
			$this->loadModel('Proposal');
			$item_report_data = $this->Proposal->find('all',array(
			'fields' => array(
				'Proposal.id',
				'Agency.id',
				'Agency.name',
				'Project.name',
				'Project.abc_amount',
				'Project.dai_amount_offered',
				'ProjectStatus.id',
				'ProjectStatus.name',
				'Proposal.quantity',
				'Item.brand',
				'Item.description',
				'Item.unit_of_measurement',
				'Proposal.unit_price',
				'Proposal.total',
				'Proposal.total',
			),
			'contain' => array(
				'Item',
				'Project'
			),
			'joins' => array(
				array(
					'table' => 'agencies',
					'alias' => 'Agency',
					'type' => 'LEFT',
					'conditions' =>'Agency.`id` = Project.`agency_id`'
				),
				array(
					'table' => 'setting_project_statuses',
					'alias' => 'ProjectStatus',
					'type' => 'LEFT',
					'conditions' =>'ProjectStatus.`id` = Project.`project_status_id`'
				),
			),
			'conditions'=> $this->default_conditions,	
			'recursive' => -1,
			));
			
			$item_report_data = reorder(array(
				'fields' => array(
					// 'Proposal.id',
					'Agency.name',
					'Project.name',
					'ProjectStatus.name',
					'Project.abc_amount_currency',
					'Project.dai_amount_offered_currency',
					'Item.brand',
					'Item.description',
					'Proposal.quantity',
					'Item.unit_of_measurement',
					'Proposal.unit_price',
					'Proposal.total',
				),
				'data' => $item_report_data
			));
			if(isset($report_format) && $report_format == 'csv') {
				$this->Export->export_csv(array(
					'title' => 'Item Report List',
					'header_data' => array(
						'Agency Name',
						'Project Name',
						'Project Status',
						'ABC Amount',
						'DAI Amount Offered',
						'Brand',
						'Description',
						'Quantity',
						'Unit',
						'Unit Price',
						'Total',
					),
					'data' => $item_report_data
				));
			}
			else if(isset($report_format) && $report_format == 'excel') {
				$this->Export->export_excel_with_template(array(
					'title' => 'Item Report List',
					'header_data' => array(
						'Agency Name',
						'Project Name',
						'Project Status',
						'ABC Amount',
						'DAI Amount Offered',
						'Brand',
						'Description',
						'Quantity',
						'Unit',
						'Unit Price',
						'Total',
					),
					'data' => $item_report_data
				));
				
			}
			else if(isset($report_format) && $report_format == 'pdf') {
				$this->Export->export_pdf_with_template(array(
					'title' => 'Item Report List',
					'header_data' => array(
						'Agency Name',
						'Project Name',
						'Project Status',
						'ABC Amount',
						'DAI Amount Offered',
						'Brand',
						'Description',
						'Quantity',
						'Unit',
						'Unit Price',
						'Total',
					),
					'data' => $item_report_data
				));
			}
		}
		else{
			$this->loadModel('Proposal');
			$this->Paginator->settings = array(
			'fields' => array(
				'Project.id',
				'Agency.id',
				'Agency.name',
				'Project.name',
				'Project.abc_amount',
				'Project.dai_amount_offered',
				'ProjectStatus.id',
				'ProjectStatus.name',
				'Proposal.quantity',
				'Item.brand',
				'Item.description',
				'Item.unit_of_measurement',
				'Proposal.unit_price',
				'Proposal.total',
			),
			'contain' => array(
				'Item',
				'Project'
			),
			'joins' => array(
				array(
					'table' => 'agencies',
					'alias' => 'Agency',
					'type' => 'LEFT',
					'conditions' =>'Agency.`id` = Project.`agency_id`'
				),
				array(
					'table' => 'setting_project_statuses',
					'alias' => 'ProjectStatus',
					'type' => 'LEFT',
					'conditions' =>'ProjectStatus.`id` = Project.`project_status_id`'
				),
			),
			'conditions'=> $this->default_conditions,	
			'recursive' => -1,
			);
			
			$item_report_data = $this->Paginator->paginate('Proposal');
			
			$item_report_data = reorder(array(
				'fields' => array(
					'Project.id',
					'Agency.name',
					'Project.name',
					'ProjectStatus.name',
					'Project.abc_amount_currency',
					'Project.dai_amount_offered_currency',
					'Item.brand',
					'Item.description',
					'Proposal.quantity',
					'Item.unit_of_measurement',
					'Proposal.unit_price',
					'Proposal.total',
				),
				'data' => $item_report_data
			));
		}
		
		if(!$item_report_data){
			$this->Alert->display('no_records_found');
		}
		$this->set('item_report_data', $item_report_data);
		// debug($item_report_data);
		// exit();
	}
	
	private function get_project_history($submitted_conditions = array(), $is_paginated = true){	
		$search_form_name = 'SearchProjectHistory';
		// debug($this->request->data);
		// exit();
		$this->loadModel('ProjectHistory');
		$this->default_conditions = array();
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions);
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_range('ProjectHistory.date_and_time', $search_form_name, 'date'));	
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('ProjectHistory.posted_by', $search_form_name, 'posted_by'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('ProjectHistory.description', $search_form_name, 'description'));
		
		$project_history_fields = array(
			'ProjectHistory.id',
			'ProjectHistory.date_and_time',
			'ProjectHistory.description',
			'ProjectHistory.item_history_description',
			'CreatedBy.first_name',
			'CreatedBy.middle_name',
			'CreatedBy.last_name',
			'CONCAT(CreatedBy.first_name, " ", CreatedBy.middle_name, " ", CreatedBy.last_name) as standard_name'
		);
		
		return $this->DaiSharedFunction->get_records(
			$this->ProjectHistory,
			$project_history_fields,
			$project_history_fields,
			array(
				'ProjectHistory.date_and_time', 
				'ProjectHistory.description', 
				'ProjectHistory.item_history_description', 
				'CONCAT(CreatedBy.first_name, " ", CreatedBy.middle_name, " ", CreatedBy.last_name) as standard_name'
			),
			array(
				'ProjectHistory.date_and_time', 
				'ProjectHistory.description', 
				'ProjectHistory.item_history_description', 
				'CONCAT(CreatedBy.first_name, " ", CreatedBy.middle_name, " ", CreatedBy.last_name) as standard_name'
			),
			array('CreatedBy'),
			$this->default_conditions,
			array('ProjectHistory.created' => 'DESC'),
			$this->default_recursive,
			$is_paginated
		);
	}
	
	
	public function bidding_export_excel_project_history_log($id) { 
		$header_for_history = array(
			'Date and Time',
			'Description',
			'Posted By'
		);
		$records = $this->get_project_history(array('ProjectHistory.project_id' => $id), false); 
		$records = convert_array(array(
			'fields' => array(
				'ProjectHistory.date_and_time' => 'F j, Y g:i A'
			),
			'data' => $records
		));
		
		$this->Export->export_excel(array(
			'title' => 'Project History List [Total Record/s - ' . count($records) . ']',
			'header_data' => $header_for_history,
			'data' => $records
		));
	}

	
	
	public function bidding_logout(){
		return $this->redirect($this->Auth->logout());
	}
	
	function beforeFilter(){
		parent::beforeFilter();
		$this->Auth->allow('bidding_logout');
	}
 
	public function isAuthorized($project) {
		return $this->DaiSharedFunction->check_project_privileges($project);
	}
}
?>