<?php
class TicketsController extends AppController {
	public $components = array('DaiSharedFunction');
  
	public $pagination_fields = array(
		'Ticket.id',
		'Ticket.date',
		'Ticket.ticket_number',
		'TicketProduct.product_name', 
		'Ticket.concern_type_code', 
		'Ticket.support_type', 
		'TicketType.name', 
		'Ticket.contact_name',
		'Ticket.company_name',
		'Ticket.ticket_status',
	);
	
	public $pagination_fields_reorder = array(
		'Ticket.id',
		'Ticket.date',
		'Ticket.ticket_number',
		'TicketProduct.product_name', 
		'Ticket.support_type', 
		'TicketType.name', 
		'Ticket.contact_name',
		'Ticket.company_name',
		'Ticket.ticket_status',
	);
			
	public $report_fields = array(
		'Ticket.id',
		'Ticket.date',
		'Ticket.ticket_number',
		'TicketProduct.product_name', 
		'Ticket.concern_type_code', 
		'Ticket.support_type', 
		'TicketType.name', 
		'Ticket.contact_name',
		'Ticket.company_name',
		'Ticket.ticket_status',
	);
	
	public $report_fields_reorder = array(  
		'Ticket.date',
		'Ticket.ticket_number',
		'TicketProduct.product_name', 
		'Ticket.support_type', 
		'TicketType.name', 
		'Ticket.contact_name',
		'Ticket.company_name',
		'Ticket.ticket_status',
	);
	
	public $default_contain = array('TicketType', 'TicketProduct');
	
	public $default_order = array(
		'Ticket.date' => 'DESC', 
		'Ticket.time' => 'DESC'
	);
	
	public $default_conditions = array( 
			'Ticket.ticket_status' => array(
				'On-Going', 
				'On-Hold'
			)
	);
	
	public $default_recursive = -1;
	
	public $default_header_data = array(
		'Date',
		'Ticker Number',
		'Product',
		'Support Type',
		'Concern Type',
		'Contact Name',
		'Company Name',
		'Ticket Status',
	);
	
	public $default_header_data_for_logs = array(
		'Date and Time',
		'Details',  
		'Modified By',  
	);
	
	public $return_address = array('controller' => 'tickets', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_success = array('controller' => 'tickets', 'action' => 'view_ticket', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_add_no_layout = array('controller' => 'tickets', 'action' => 'add_ticket', 'prefix' => 'bidding', 'bidding' => true, '?' => array('no-layout' => 1));
	
	public $search_form_name = 'SearchTicket';
	
	public $singular_alert_object = 'Ticket';
	
	public $plural_alert_object = 'Tickets';
	
	public $singular_set_name = 'ticket';
	
	public $plural_set_name = 'tickets';
		

	public function bidding_index(){ 
		$records = $this->get_tickets(); 
		if(!$records) {
			$this->Alert->display('no_records_found');
		}
		
		$this->DaiSharedFunction->get_list_from_model('TicketType', 'ticket_type_list', 'find_all_list');
		$this->DaiSharedFunction->get_list_from_model('TicketProduct', 'ticket_product_list', 'find_all_list');
		
		$this->set($this->plural_set_name, $records);
	}
		
	public function bidding_export_csv_ticket() { 
		
		$records = $this->get_tickets(
			array(),
			false
		);
		
		$this->Export->export_csv(array(
			'title' => 'Ticket List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_excel_ticket() { 
		
		$records = $this->get_tickets(
			array(),
			false
		);
		
		$this->Export->export_excel(array(
			'title' => 'Ticket List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_pdf_ticket() { 
		
		$records = $this->get_tickets(
			array(),
			false
		);
		
		$this->Export->export_pdf(array(
			'title' => 'Ticket List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}

	public function bidding_export_csv_ticket_history_log($id) { 
		
		$records = $this->get_ticket_history_logs(array('TicketHistory.ticket_id' => $id), false); 
		$records = convert_array(array(
			'fields' => array(
				'TicketHistory.created' => 'F j, Y g:i A'
			),
			'data' => $records
		));
		
		$this->Export->export_csv(array(
			'title' => 'Ticket Log List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data_for_logs,
			'data' => $records
		));
	}
	
	public function bidding_export_excel_ticket_history_log($id) { 
		
		$records = $this->get_ticket_history_logs(array('TicketHistory.ticket_id' => $id), false); 
		$records = convert_array(array(
			'fields' => array(
				'TicketHistory.created' => 'F j, Y g:i A'
			),
			'data' => $records
		));
		
		$this->Export->export_excel(array(
			'title' => 'Ticket Log List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data_for_logs,
			'data' => $records
		));
	}

	public function bidding_export_pdf_ticket_history_log($id) { 
		
		$records = $this->get_ticket_history_logs(array('TicketHistory.ticket_id' => $id), false); 
		$records = convert_array(array(
			'fields' => array(
				'TicketHistory.created' => 'F j, Y g:i A'
			),
			'data' => $records
		));
		
		$this->Export->export_pdf(array(
			'title' => 'Ticket Log List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data_for_logs,
			'data' => $records
		));
	}

	public function bidding_on_going_ticket($id = null){
		
		if ($id) {
			$this->autoRender = false; 
			$ticket_data = $this->Ticket->find_by_id($id);
			if ($ticket_data) {
				$data = array();
				$data['Ticket']['id'] = $id;
				$data['Ticket']['ticket_status'] = 'On-Going'; 
				$data['TicketHistory'][0] = $ticket_data['Ticket'];
				$data['TicketHistory'][0]['ticket_status'] = 'On-Going';
				$data['TicketHistory'][0]['details'] = 'Ticket has been set as on-going';
				
				unset($data['TicketHistory'][0]['id']);
				unset($data['TicketHistory'][0]['created']);
				unset($data['TicketHistory'][0]['modified']); 
				unset($data['TicketHistory'][0]['status']);
				unset($data['TicketHistory'][0]['created_by']);
				unset($data['TicketHistory'][0]['modified_by']);
		 
				if($this->Ticket->saveAssociated($data, array('deep' => true))){
					$this->Alert->display('success_ticket_on_going');
					return $this->redirect(array('controller' => 'tickets', 'action' => 'view_ticket', $id,'prefix' => 'bidding', 'bidding' => true));
				}
				else{ 
					$this->Alert->display('failed_ticket');
					return $this->redirect(array('controller' => 'tickets', 'action' => 'view_ticket', $id,'prefix' => 'bidding', 'bidding' => true));
				}
			}
			else {
				$this->Alert->display('no_records_found');
				return $this->redirect($this->return_address);
			}
			
		}
		else {
			$this->Alert->display('no_records_found');
			return $this->redirect($this->return_address);
		}
		
	}		
		
	public function bidding_on_hold_ticket($id = null){
		
		if ($id) { 
			$this->autoRender = false; 
			$ticket_data = $this->Ticket->find_by_id($id);
			if ($ticket_data) {
				$data = array();
				$data['Ticket']['id'] = $id;
				$data['Ticket']['ticket_status'] = 'On-Hold'; 
				$data['TicketHistory'][0] = $ticket_data['Ticket'];
				$data['TicketHistory'][0]['ticket_status'] = 'On-Hold'; 
				$data['TicketHistory'][0]['details'] = 'Ticket has been set as on-hold';
				unset($data['TicketHistory'][0]['id']);
				unset($data['TicketHistory'][0]['created']);
				unset($data['TicketHistory'][0]['modified']); 
				unset($data['TicketHistory'][0]['status']);
				unset($data['TicketHistory'][0]['created_by']);
				unset($data['TicketHistory'][0]['modified_by']);
			 
				if($this->Ticket->saveAssociated($data, array('deep' => true))) {
					$this->Alert->display('success_ticket_on_hold');
					return $this->redirect(array('controller' => 'tickets', 'action' => 'view_ticket', $id,'prefix' => 'bidding', 'bidding' => true));
				}
				else{
					$this->Alert->display('failed_ticket');
					return $this->redirect(array('controller' => 'tickets', 'action' => 'view_ticket', $id,'prefix' => 'bidding', 'bidding' => true));
				}
			}
			else {
				$this->Alert->display('no_records_found');
				return $this->redirect($this->return_address);
			}
		
		}
		else {
			$this->Alert->display('no_records_found');
			return $this->redirect($this->return_address);
		}
		
	}		
		
	public function bidding_close_ticket($id = null){
		
		if ($id) {
			$this->autoRender = false; 
			$ticket_data = $this->Ticket->find_by_id($id);
			if ($ticket_data) {
				$data = array();
				$data['Ticket']['id'] = $id;
				$data['Ticket']['ticket_status'] = 'Closed'; 
				$data['TicketHistory'][0] = $ticket_data['Ticket'];
				$data['TicketHistory'][0]['ticket_status'] = 'Closed'; 
				$data['TicketHistory'][0]['details'] = 'Ticket has been closed';
				unset($data['TicketHistory'][0]['id']);
				unset($data['TicketHistory'][0]['created']);
				unset($data['TicketHistory'][0]['modified']); 
				unset($data['TicketHistory'][0]['status']);
				unset($data['TicketHistory'][0]['created_by']);
				unset($data['TicketHistory'][0]['modified_by']);

				if($this->Ticket->saveAssociated($data, array('deep' => true))) {
					$this->Alert->display('success_ticket_closed');
					return $this->redirect(array('controller' => 'tickets', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
				}
				else{
					$this->Alert->display('failed_ticket');
					return $this->redirect(array('controller' => 'tickets', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
				}
			}
			else {
				$this->Alert->display('no_records_found');
				return $this->redirect($this->return_address);
			}
			
		}
		else {
			$this->Alert->display('no_records_found');
			return $this->redirect($this->return_address);
		}
		
	}
	
	public function bidding_view_ticket($id = null){ 
		if ($id) {
			$ticket_data = $this->Ticket->find_by_id_with_contain($id); 
			if ($ticket_data) {
				$ticket_history_logs = $this->get_ticket_history_logs(array('TicketHistory.ticket_id' => $ticket_data['Ticket']['id']));
				
				if ($this->request->is('post') || $this->request->is('put')) {
				
					$this->request->data['TicketHistory']['ticket_id'] = $id;
					$this->request->data['TicketHistory']['ticket_status'] = 'add';
					 
					if ($this->TicketHistory->save($this->request->data)){
						$this->Alert->success_add('Ticket Log');
						return $this->redirect(array('controller' => 'tickets', 'action' => 'view_ticket', $id,'prefix' => 'bidding', 'bidding' => true));
					}
					else{
						
						$this->Alert->failed_add('Ticket Log');
						return $this->redirect(array('controller' => 'tickets', 'action' => 'view_ticket', $id,'prefix' => 'bidding', 'bidding' => true));
					}
				}
				$this->set('ticket_history_logs',$ticket_history_logs);
				$this->set($this->singular_set_name,$ticket_data);
			}
			else {
				$this->Alert->display('no_records_found');
				return $this->redirect($this->return_address);
			}
			
		}
		else {
			$this->Alert->display('no_records_found');
			return $this->redirect($this->return_address);
			
		} 
	}
	
	private function get_tickets($submitted_conditions = array(), $is_paginated = true) {
		$search_form_name = $this->search_form_name;
		
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions);
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_date_equivalent('Ticket.date', $search_form_name, 'date'));  
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Ticket.ticket_number', $search_form_name, 'ticket_number'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Ticket.product_code', $search_form_name, 'product'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Ticket.support_type', $search_form_name, 'support_type'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Ticket.concern_type_code', $search_form_name, 'concern_type'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Ticket.contact_name', $search_form_name, 'contact_name'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('Ticket.company_name', $search_form_name, 'company_name'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('Ticket.ticket_status', $search_form_name, 'ticket_status'));  
 
		return $this->DaiSharedFunction->get_records(
			$this->Ticket, 
			$this->pagination_fields,
			$this->pagination_fields_reorder,
			$this->report_fields,
			$this->report_fields_reorder,
			$this->default_contain,
			$this->default_conditions,
			$this->default_order,
			$this->default_recursive,
			$is_paginated
		); 
	}
	
	private function get_ticket_history_logs($submitted_conditions = array(), $is_paginated = true) {
		$this->loadModel('TicketHistory');
		$search_form_name = $this->search_form_name;
		   
		$default_conditions = array();
		$default_conditions = array_merge($default_conditions, $submitted_conditions);
		$default_conditions = array_merge($default_conditions, $this->DaiConditionBuilder->build_string_equivalent('TicketHistory.ticket_status', $search_form_name, 'ticket_status')); 
		$default_conditions = array_merge($default_conditions, $this->DaiConditionBuilder->build_text_comparison('TicketHistory.details', $search_form_name, 'details'));  
		$default_conditions = array_merge($default_conditions, $this->DaiConditionBuilder->build_text_comparison('CONCAT(CreatedBy.first_name, " ", CreatedBy.middle_name, " ", CreatedBy.last_name)', $search_form_name, 'modified_by'));  
		$default_conditions = array_merge($default_conditions, $this->DaiConditionBuilder->build_date_equivalent('DATE_FORMAT(TicketHistory.created, "%Y-%m-%d")', $search_form_name, 'date'));   
 
		$fields = array(
			'TicketHistory.id', 
			'TicketHistory.ticket_status', 
			'TicketHistory.created', 
			'TicketHistory.details',
			'CreatedBy.first_name',
			'CreatedBy.middle_name',
			'CreatedBy.last_name',
			'CONCAT(CreatedBy.first_name, " ", CreatedBy.middle_name, " ", CreatedBy.last_name) as standard_name'
		);
		   
		return $this->DaiSharedFunction->get_records(
			$this->TicketHistory, 
			$fields,
			$fields,
			array(
				'TicketHistory.created', 
				'TicketHistory.details', 
				'CONCAT(CreatedBy.first_name, " ", CreatedBy.middle_name, " ", CreatedBy.last_name) as standard_name'
			),
			array(
				'TicketHistory.created', 
				'TicketHistory.details', 
				'CONCAT(CreatedBy.first_name, " ", CreatedBy.middle_name, " ", CreatedBy.last_name) as standard_name'
			),
			array('CreatedBy'),
			$default_conditions,
			array('TicketHistory.created' => 'DESC'),
			$this->default_recursive,
			$is_paginated
		); 
	}
	 
	}
?>