<?php
class UsersController extends AppController {
	public $components = array('DaiSharedFunction');
  
	public $pagination_fields = array(
		'User.id',
		'CONCAT(User.first_name, " ", User.middle_name, " ", User.last_name) as standard_name',
		'User.username',
		'Role.name',
		'User.lock_flag',
		'User.status'
	);
	
	public $pagination_fields_reorder = array(
		'User.id',
		'CONCAT(User.first_name, " ", User.middle_name, " ", User.last_name) as standard_name',
		'User.username',
		'Role.name',
		'User.lock_flag_name',
		'User.status_name'
	);
			
	public $report_fields = array(
		'User.id',
		'CONCAT(User.first_name, " ", User.middle_name, " ", User.last_name) as standard_name',
		'User.username',
		'Role.name',
		'User.lock_flag',
		'User.status'
	);
	
	public $report_fields_reorder = array(
		'CONCAT(User.first_name, " ", User.middle_name, " ", User.last_name) as standard_name',
		'User.username',
		'Role.name',
		'User.lock_flag_name',
		'User.status_name'
	);
	
	public $default_contain = array(
		'Role'
	);
	
	public $default_order = array('User.first_name' => 'ASC');
	
	public $default_conditions = array();
	
	public $default_recursive = -1;
	
	public $default_header_data = array(
		'Name',
		'Username',
		'Role',
		'Locked',
		'Status'
	);
	
	public $return_address = array('controller' => 'users', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_success = array('controller' => 'users', 'action' => 'view_user', 'prefix' => 'bidding', 'bidding' => true);
	
	public $return_address_on_add_no_layout = array('controller' => 'users', 'action' => 'add_user', 'prefix' => 'bidding', 'bidding' => true, '?' => array('no-layout' => 1));
	
	public $search_form_name = 'SearchUser';
	
	public $singular_alert_object = 'User';
	
	public $plural_alert_object = 'Users';
	
	public $singular_set_name = 'user';
	
	public $plural_set_name = 'users';
	
	public function bidding_index() { 
		$records = $this->get_users();
	 
		if(!$records) {
			$this->Alert->display('no_records_found');
		}
		
		$this->DaiSharedFunction->get_list_from_model('Role', 'role_list');
		$this->set($this->plural_set_name, $records);
	}
	
	public function bidding_export_csv_user() { 
		
		$records = $this->get_users(
			array(),
			false
		);
		
		$this->Export->export_csv(array(
			'title' => 'User List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_excel_user() { 
		
		$records = $this->get_users(
			array(),
			false
		);
		
		$this->Export->export_excel(array(
			'title' => 'User List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}
	
	public function bidding_export_pdf_user() { 
		$records = $this->get_users(
			array(),
			false
		);
		
		$this->Export->export_pdf(array(
			'title' => 'User List [Total Record/s - ' . count($records) . ']',
			'header_data' => $this->default_header_data,
			'data' => $records
		));
	}

	public function bidding_add_user() { 
		if($this->request->is('put') || $this->request->is('post')) {
			$data = array();
			unset($this->request->data['search_keyword']);
			
			$data = $this->request->data;
			
			if($data['User']['role_id'] == '1'){
				$data['User']['role'] = 'Administrator';
			}
			else if($data['User']['role_id'] == '2'){
				$data['User']['role'] = 'Encoder';
			}
			// debug($data);
			// exit();
			if($this->User->saveAssociated($data, array('deep' => true))){
				$this->Session->setFlash('User has been successfully added.' , 'alert.success-full');
				return $this->redirect(array('controller' => 'users', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
			else{
				$this->Session->setFlash('Failed to add. Please try again', 'alert.failed-full');
				return $this->redirect(array('controller' => 'users', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
		}
		// debug($this->request->data);
		// exit();
		$this->DaiSharedFunction->get_list_from_model('Role', 'role_list'); 
	}
	
	public function bidding_get_list() {
		return $this->DaiSharedFunction->get_list(
			$this->User
		);
	}
	
	public function bidding_edit_user($id = null) { 
		$this->DaiSharedFunction->edit_and_lock_record(
			$this->User, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
		
		$this->DaiSharedFunction->get_list_from_model('Role', 'role_list'); 
	}
	
	public function bidding_override_password($id = null) { 
		$this->DaiSharedFunction->override_record(
			$this->User, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_change_password(){ 
		$this->User->attach_model();
		
		if($this->request->is('put') || $this->request->is('post')) {
			if($this->User->change_password($this->request->data)) {
				$this->Alert->display('success_change_password');
				return $this->redirect(array('controller' => 'home', 'action' => 'index', 'prefix' => 'bidding', 'bidding' => true));
			}
			else {
				$this->Alert->display('failed_change_password');
			}
		}
	}
	
	public function bidding_view_user($id = null) { 
		$this->DaiSharedFunction->view_and_lock_record(
			$this->User, 
			$this->return_address,
			$this->singular_set_name,
			$id
		);
	}
	
	public function bidding_enable_user($id = null) {  
		$this->DaiSharedFunction->enable_record(
			$this->User, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_disable_user($id = null) { 
		
		$this->DaiSharedFunction->disable_record(
			$this->User, 
			array_merge($this->return_address_on_success, array($id)),
			$this->return_address,
			$id
		);
	}
	
	public function bidding_delete_user($id = null) { 
		$this->DaiSharedFunction->delete_record(
			$this->User, 
			$this->return_address,
			$id
		);
	}
	
	private function get_users($submitted_conditions = array(), $is_paginated = true) {
		$search_form_name = $this->search_form_name;
		
		$this->default_conditions = array();
		$this->default_conditions = array_merge($this->default_conditions, $submitted_conditions);
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison(array('User.first_name','User.middle_name','User.last_name'), $search_form_name, 'name'));   
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_text_comparison('User.username', $search_form_name, 'username')); 
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('User.role_id', $search_form_name, 'role_id'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_string_equivalent('User.lock_flag', $search_form_name, 'locked'));
		$this->default_conditions = array_merge($this->default_conditions, $this->DaiConditionBuilder->build_status('User.status', $search_form_name, 'status'));
 
		return $this->DaiSharedFunction->get_records(
			$this->User, 
			$this->pagination_fields,
			$this->pagination_fields_reorder,
			$this->report_fields,
			$this->report_fields_reorder,
			$this->default_contain,
			$this->default_conditions,
			$this->default_order,
			$this->default_recursive,
			$is_paginated
		); 
	}
	
	public function bidding_logout(){
		return $this->redirect($this->Auth->logout());
	}
	
	function beforeFilter(){
		parent::beforeFilter();
		$this->Auth->allow('bidding_logout');
	}
 
	public function isAuthorized($user) {
		return $this->DaiSharedFunction->check_user_privileges($user);
	}
}
?>