<?php
App::uses('AppModel', 'Model');
App::uses('SimplePasswordHasher', 'Controller/Component/Auth');

class Agency extends AppModel { 
	public $actsAs = array('DaiSharedFunction', 'DaiSharedCallback');
	
	public $audit_model = 'Agency';
	public $audit_module = 'Agency';
	public $audit_singular_alias = 'Agency';
	public $audit_plural_alias = 'Agencys';
	
	public $validate = array(
        'id' => array(
			'validation 1' => array(
				'required' => 'update',
				'rule' => array('notBlank'),
				'message'	=> 'ID is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'ID must be a non-decimal number that is greater than zero.'
			),
			'validation 3' => array(
				'rule' => array('isUnique'),
				'message'	=> 'ID is already existing.'
			)
        ),
		'status' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Status is required and must not be empty.'
			)
        ), 
		'created_by' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Created by is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'Please supply a valid Created by' 
			)
        ), 
	);
	 
	
	public $belongsTo = array( 
		'CreatedBy' => array(
			'className' => 'User',
			'foreignKey' => 'created_by'
		),
		'LastModifiedBy' => array(
			'className' => 'User',
			'foreignKey' => 'modified_by'
		),
	);
	
	
	public function find_by_id($id) {
		$results = $this->find('first', array(
			'contain' => array(
				'CreatedBy',
				'LastModifiedBy'
			),
			'conditions' => array(
				'Agency.id' => $id,
			),
		));
		
		if(!empty($results)) {
			return $results;
		}
	 
		return false;
	}
	
	
	public function beforeValidate($options = array()) {
		if(!isset($this->data['Agency']['id'])) {
			$this->data['Agency']['status'] = '1';
		}
	}
	
	// public function afterValidate() { 
	
	// }
}
