<?php
App::uses('AppModel', 'Model');
App::uses('SimplePasswordHasher', 'Controller/Component/Auth');

class Attachment extends AppModel { 
	public $actsAs = array('DaiSharedFunction', 'DaiSharedCallback');
	
	
	public $tablePrefix = 'project_';
	public $audit_model = 'Attachment';
	public $audit_module = 'Attachment';
	public $audit_singular_alias = 'Attachment';
	public $audit_plural_alias = 'Attachments';
	
	public $validate = array(
        'id' => array(
			'validation 1' => array(
				'required' => 'update',
				'rule' => array('notBlank'),
				'message'	=> 'ID is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'ID must be a non-decimal number that is greater than zero.'
			),
			'validation 3' => array(
				'rule' => array('isUnique'),
				'message'	=> 'ID is already existing.'
			)
        ),
		'status' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Status is required and must not be empty.'
			)
        ), 
		'created_by' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Created by is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'Please supply a valid Created by' 
			)
        ), 
	);
	 
	
	// public $belongsTo = array(
		// 'Item' => array(
			// 'className' => 'Item',
			// 'foreignKey' => 'item_id',
		// ),
	// );
	
	
	public function find_by_id($id) {
		$results = $this->find('first', array(
			'conditions' => array(
				'Attachment.id' => $id,
			),
		));
		
		if(!empty($results)) {
			return $results;
		}
	 
		return false;
	}
	
	public function afterFind($results, $primary = false) { 
		 
		foreach($results as $results_key => $results_value) {    
			if (isset($results_value['Attachment']['date_uploaded'])) {
				$results[$results_key]['Attachment']['date_uploaded'] = convert_datetime($results_value['Attachment']['date_uploaded'], 'm/d/Y'); 
			}  
		} 	 
		return $results;
	}
	
	public function beforeValidate($options = array()) {
		if(!isset($this->data['Attachment']['id'])) {
			$this->data['Attachment']['status'] = '1';
		}
		if(is_array($this->data['Attachment']['attachment']) && $this->data['Attachment']['attachment']['size'] > 0){
			if(!isset($this->data['Attachment']['date_uploaded'])) {
				$this->data['Attachment']['date_uploaded'] = date('Y-m-d');
			}
		}
	}
	
	public function afterValidate() { 
		
		$this->data[$this->name] = prepare_date_format_for_saving($this->data[$this->name], array('date_uploaded'));
		
		if(isset($this->data['Attachment']['attachment'])) {
			if(is_array($this->data['Attachment']['attachment']) && $this->data['Attachment']['attachment']['size'] > 0){
				$new_image_icon = $this->data['Attachment']['attachment'];
				$icon_path = "files". "/" ."image/Attachment"; // DIRECTORY
				
				$temp = explode(".", $new_image_icon["name"]);
				$icon_name = uniqid().rand(0,99) . '.' . end($temp);
				if (!file_exists($icon_path)) 
				{
					mkdir($icon_path, 0777, true);
				}
				move_uploaded_file( $new_image_icon["tmp_name"],$icon_path. "/" . $icon_name );
				$new_image_icon['attachment'] = $icon_path. "/" . $icon_name;
				
				$this->data['Attachment']['attachment'] = $new_image_icon['attachment'];
			}
			else {
				unset($this->data['Attachment']['attachment']);
			}
		}
	
	}
}
