<?php
class DaiSharedFunctionBehavior extends ModelBehavior {
	public function add_record(&$model, $data = array()) {
		if(isset($data[$model->name][$model->displayField])) {
			$existing_data = $this->find_by_display_field($model, $data[$model->name][$model->displayField]);
		}
		else {
			$existing_data = false;
		}
		
		if($existing_data) {
			$existing_data[$model->name]['status'] = 1;
			$data = $existing_data;
		}

		return $model->saveAssociated($data, array('deep' => true)); 
	}
	
	public function update_record(&$model, $data) {
		return $model->saveAssociated($data, array('deep' => true));
	}
	
	public function delete_record(&$model, $id) {
		$existing_data = $this->find_by_id($model, $id);
		
		if($existing_data && $existing_data[$model->name]['lock_flag'] == 0) {	
			$lock_primary = false;		
			
			$delete_result = !$this->is_already_used($model, $id, true, true);
			
			return $delete_result;
		}
		
		return false;
	}
	
	public function is_already_used(&$model, $id, $proceed_with_delete = false, $proceed_to_lock = false) {
		$primary_model_datasource = $model->getDataSource();
		$primary_model_datasource->begin();
		
		try {
			$is_already_used = !$model->delete($id);
		} 
		catch (Exception $e) {
			$primary_checking = (isset($e->errorInfo[2]) ? $e->errorInfo[2] : '');

			if(strpos($primary_checking, 'foreign key constraint fails') !== false) {
				$is_already_used = true;
			}
			else {
				$is_already_used = false;
			}
		}
		
		if($proceed_with_delete) {
			$primary_model_datasource->commit();
		}
		else {
			$primary_model_datasource->rollback();
			$primary_model_datasource->commit();
		}
		
		$primary_model_datasource->begin();
		
		if($is_already_used) {
			if($proceed_to_lock) {
				try {
					if($model->lock_record($id)) {
						$is_already_used = true;
					}
				} 
				catch (Exception $e) {
					$primary_model_datasource->rollback();
				}
			}
		}
		
		$primary_model_datasource->commit();
		
		return $is_already_used;
	}
	
	public function deep_delete_record(&$model, $id) {
		$existing_data = $this->find_by_id($model, $id);
		
		if($existing_data && $existing_data[$model->name]['lock_flag'] == 0) {
			$is_success = true;
			$registered_model_delete_result = array();
			$delete_result = true;
			$lock_primary = false;
			$lock_secondary = array();
			
			$this->get_dependent_model_mapping($model->name, $related_models);
			$registered_models = array();
			$registered_data_source = array();
			
			$primary_model_datasource = $model->getDataSource();
			$primary_model_datasource->begin();
			
			foreach($related_models as $related_model_name => $related_model_value) {
				$registered_models[$related_model_name] = ClassRegistry::init($related_model_name);
				$registered_data_source[$related_model_name] = $registered_models[$related_model_name]->getDataSource();
				$registered_data_source[$related_model_name]->begin();
				
				try {
					$registered_model_delete_result[$related_model_name] = $registered_models[$related_model_name]->deleteAll(array(
						$related_model_name . '.' . $related_model_value['foreignKey'] => $id
					), false);
					
					$is_success = $is_success && $registered_model_delete_result[$related_model_name];
					$lock_secondary[$related_model_name] = !$is_success;
				} 
				catch (Exception $e) {
					$is_success = false;
					
					$registered_model_delete_result[$related_model_name] = $is_success;
					$lock_secondary[$related_model_name] = !$is_success;
				}
			}
			
			try {
				$delete_result = $model->delete($id, false);
			} 
			catch (Exception $e) {
				$primary_checking = (isset($e->errorInfo[2]) ? $e->errorInfo[2] : '');
				if(strpos($primary_checking, 'foreign key constraint fails') !== false) {
					$lock_primary = true;
				}

				$delete_result = false;
			}
			
			if($is_success && $delete_result) {
				foreach($related_models as $related_model_name => $related_model_value) {
					$registered_data_source[$related_model_name]->commit();
					$primary_model_datasource->commit();
				}
				
				return true;
			}
			else {
				foreach($related_models as $related_model_name => $related_model_value) {
					$registered_data_source[$related_model_name]->rollback();
					$registered_data_source[$related_model_name]->commit();
				}
				
				$primary_model_datasource->rollback();
				$primary_model_datasource->commit();
				
				
				if($lock_primary) {
					$primary_model_datasource->begin();
					
					try {
						if($model->lock_record($id)) {
							$primary_model_datasource->commit();
						}
						else {
							$primary_model_datasource->rollback();
							$primary_model_datasource->commit();
						}
					} 
					catch (Exception $e) {
						
						$primary_model_datasource->rollback();
						$primary_model_datasource->commit();
					}
				}
				
				return false;
			}
		}
		
		return false;
	}
	
	private function get_dependent_model_mapping($system_model, &$model_mapping) {
		$selected_model = ClassRegistry::init($system_model);
		
		foreach($selected_model->hasOne as $model_key => $model_value) {
			if(isset($model_value['className'])) {
				$model_mapping_key = $model_value['className'];
			}
			else {
				$model_mapping_key = $model_key;
			}
			
			if(isset($model_value['dependent']) && $model_value['dependent']) {
				if(!isset($model_mapping[$model_mapping_key]) || !in_array($system_model, $model_mapping[$model_mapping_key])) {
					$model_mapping[] = $model_mapping_key;
				}
			}
		}
		
		foreach($selected_model->hasMany as $model_key => $model_value) {
			if(isset($model_value['className'])) {
				$model_mapping_key = $model_value['className'];
			}
			else {
				$model_mapping_key = $model_key;
			}
			
			
			if(isset($model_value['dependent']) && $model_value['dependent']) {
				if(!isset($model_mapping[$model_mapping_key]) || !in_array($system_model, $model_mapping[$model_mapping_key])) {
					$model_mapping[$model_mapping_key] = $model_value;
				}
			}
		}
		
		return $model_mapping;
	}
	
	public function find_by_display_field(&$model, $display_field) {
		$default_conditions = array(
			$model->displayField => $display_field
		);
		
		$result = $model->find('first', array(
			'fields' => array(
				$model->primaryKey,
				$model->displayField,
				'status',
				'lock_flag'
			),
			'conditions' => $default_conditions,
			'recursive' => -1
		));
		
		return $result;
	}
	
	public function find_by_primary_key(&$model, $primary_key) {
		$result = $model->find('first', array(
			'fields' => array(
				$model->primaryKey,
				$model->displayField,
				'status',
				'lock_flag'
			),
			'conditions' =>array(
				$model->primaryKey => $primary_key
			),
			'recursive' => -1
		));
		
		return $result;
	}
	
	public function find_by_id(&$model, $id) {
		$result = $this->find_by_primary_key($model, $id);
		
		return $result;
	}
	
	public function get_list(&$model) {
		$result = $model->find('list', array(
			'fields' => array(
				$model->primaryKey,
				$model->displayField
			),
			'conditions' => array(
				'status' => 1
			),
			'order' => array(
				$model->displayField . ' ASC'
			)
		));
		
		return $result;
	}
	
	public function enable_record(&$model, $id) {
		return $model->update_status($id, 1);
	}
	
	public function disable_record(&$model, $id) {
		return $model->update_status($id, 0);
	}
	
	public function update_status(&$model, $id, $status) {
		$data[$model->name]['id'] = $id;
		$data[$model->name]['status'] = $status;
		
		return $model->save($data);
	}
	
	public function unmonitor_record(&$model, $id) {
		return $model->update_is_monitored($id, 0);
	}
	
	public function monitor_record(&$model, $id) {
		return $model->update_is_monitored($id, 1);
	}
	
	public function update_is_monitored(&$model, $id, $is_monitored) {
		$data[$model->name]['id'] = $id;
		$data[$model->name]['is_monitored'] = $is_monitored;
		
		return $model->save($data);
	}
	
	public function uninclude_in_alert_record(&$model, $id) {
		return $model->update_is_include_in_alert($id, 0);
	}
	
	public function include_in_alert_record(&$model, $id) {
		return $model->update_is_include_in_alert($id, 1);
	}
	
	public function update_is_include_in_alert(&$model, $id, $include_in_alerts) {
		$data[$model->name]['id'] = $id;
		$data[$model->name]['include_in_alerts'] = $include_in_alerts;
		
		return $model->save($data);
	}
	
	public function lock_record(&$model, $id) {
		return $model->update_lock_flag($id, 1);
	}
	
	public function unlock_record(&$model, $id) {
		return $model->update_lock_flag($id, 0);
	}
	
	public function update_lock_flag(&$model, $id, $lock_flag) {
		$data[$model->name]['id'] = $id;
		$data[$model->name]['lock_flag'] = $lock_flag;
		
		return $model->save($data);
	}
	
	public function post_record(&$model, $id) {
		return $model->update_post_status($id, 1);
	}
	
	public function update_post_status(&$model, $id, $is_posted) {
		$data[$model->name]['id'] = $id;
		$data[$model->name]['is_posted'] = $is_posted;
		$data[$model->name]['posted_on'] = date('Y-m-d H:i:s');
		$data[$model->name]['posted_by'] = $_SESSION['Auth']['User']['id']; 
		$data[$model->name]['lock_flag'] = 1; 
		
		return $model->save($data);
	}
}
