<?php
App::uses('AppModel', 'Model');
App::uses('SimplePasswordHasher', 'Controller/Component/Auth');

class Project extends AppModel { 
	public $actsAs = array('DaiSharedFunction', 'DaiSharedCallback');
	
	
	// public $tablePrefix = 'setting_';
	public $audit_model = 'Project';
	public $audit_module = 'Project';
	public $audit_singular_alias = 'Project';
	public $audit_plural_alias = 'Projects';
	
	public $validate = array(
        'id' => array(
			'validation 1' => array(
				'required' => 'update',
				'rule' => array('notBlank'),
				'message'	=> 'ID is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'ID must be a non-decimal number that is greater than zero.'
			),
			'validation 3' => array(
				'rule' => array('isUnique'),
				'message'	=> 'ID is already existing.'
			)
        ),
		'status' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Status is required and must not be empty.'
			)
        ), 
		'created_by' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Created by is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'Please supply a valid Created by' 
			)
        ), 
	);
	
	public $belongsTo = array(
		'Agency' => array(
			'className' => 'Agency',
			'foreignKey' => 'agency_id'
		),
		'ProjectStatus' => array(
			'className' => 'ProjectStatus',
			'foreignKey' => 'project_status_id'
		),
	);
	 
	
	public $hasMany = array(
		'Proposal' => array(
			'className' => 'Proposal',
			'foreignKey' => 'project_id',
		),
		'Attachment' => array(
			'className' => 'Attachment',
			'foreignKey' => 'project_id',
		),
		'ProjectHistory' => array(
			'className' => 'ProjectHistory',
			'foreignKey' => 'project_id',
		),
	);
	
	
	public function find_by_id($id) {
		$results = $this->find('first', array(
			'conditions' => array(
				'Project.id' => $id,
			),
		));
		
		if(!empty($results)) {
			return $results;
		}
	 
		return false;
	}
	
	public function afterFind($results, $primary = false) { 
		 
		foreach($results as $results_key => $results_value) {    
			if (isset($results_value['Project']['date_downloaded'])) {
				$results[$results_key]['Project']['date_downloaded'] = convert_datetime($results_value['Project']['date_downloaded'], 'm/d/Y'); 
			} 
			
			if (isset($results_value['Project']['deadline_of_submission'])) {
				$results[$results_key]['Project']['deadline_of_submission'] = convert_datetime($results_value['Project']['deadline_of_submission'], 'm/d/Y'); 
			}  
		
			if (isset($results_value['Project']['delivery_period'])) {
				$results[$results_key]['Project']['delivery_period'] = convert_datetime($results_value['Project']['delivery_period'], 'm/d/Y'); 
			} 
			
			if (isset($results_value['Project']['pre_bidding_date'])) {
				$results[$results_key]['Project']['pre_bidding_date'] = convert_datetime($results_value['Project']['pre_bidding_date'], 'm/d/Y'); 
			} 
			
			if (isset($results_value['Project']['target_delivery_date'])) {
				$results[$results_key]['Project']['target_delivery_date'] = convert_datetime($results_value['Project']['target_delivery_date'], 'm/d/Y'); 
			} 
			
			if (isset($results_value['Project']['delivered_on'])) {
				$results[$results_key]['Project']['delivered_on'] = convert_datetime($results_value['Project']['delivered_on'], 'm/d/Y'); 
			} 
			
			if (isset($results_value['Project']['abc_amount'])) {
				$results[$results_key]['Project']['abc_amount_currency'] = convert_to_currency($results_value['Project']['abc_amount']); 
			} 
			
			if (isset($results_value['Project']['dai_amount_offered'])) {
				$results[$results_key]['Project']['dai_amount_offered_currency'] = convert_to_currency($results_value['Project']['dai_amount_offered']); 
			} 
			
			if (isset($results_value['Project']['winning_proposal_amount'])) {
				$results[$results_key]['Project']['winning_proposal_amount_currency'] = convert_to_currency($results_value['Project']['winning_proposal_amount']); 
			} 
			
		} 	 
		return $results;
	}
	
	public function beforeValidate($options = array()) {
		if(!isset($this->data['Project']['id'])) {
			$this->data['Project']['status'] = '1';
		}
	}
	
	public function afterValidate() { 
		$this->data[$this->name] = prepare_date_format_for_saving($this->data[$this->name], array('date_downloaded','deadline_of_submission','pre_bidding_date', 'target_delivery_date', 'delivered_on'));
	}
}
