<?php
App::uses('Model', 'Model');

class Role extends AppModel {
	public $tablePrefix = 'acc_';
	
	public $actsAs = array('DaiSharedFunction', 'DaiSharedCallback');
	
	public $validate = array(
        'id' => array(
			'validation 1' => array(
				'required' => 'update',
				'rule' => array('notBlank'),
				'message'	=> 'ID is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'ID must be a non-decimal number that is greater than zero.'
			),
			'validation 3' => array(
				'rule' => array('isUnique'),
				'message'	=> 'ID is already existing.'
			)
        ),
		'name' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Name is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('maxLength', 30),
				'message'	=> 'Name must be no longer than 30 characters.'
			),
			'validation 3' => array(
				'rule' => array('isUnique'),
				'message'	=> 'Name is already existing.'
			)
        ),
		'layout' => array(
			'validation 1' => array( 
				'rule' => array('notBlank'),
				'message'	=> 'Layout is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('maxLength', 30),
				'message'	=> 'Layout must be no longer than 30 characters.'
			)
        ),
		'status' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Status is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('boolean'),
				'message'	=> 'Status must be a boolean.'
			)
        ),
		'lock_flag' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Lock flag must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('boolean'),
				'message'	=> 'Lock flag must be a boolean.'
			)
        ),
		'created_by' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Created by is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'Please supply a valid Last Created by'  
			)
        ),
		'last_modified_by' => array(
			'validation 1' => array(
				'required' => true,
				'rule' => array('notBlank'),
				'message'	=> 'Last modified by is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'Please supply a valid Last Modified by'  
			)
        )
	);
	
	public $belongsTo = array(  
		'CreatedBy' => array(
			'className' => 'User', 
			'foreignKey' => 'created_by',
			'fields' => array(
				'CreatedBy.id',
				'CreatedBy.role_id',
				'CreatedBy.status',
			) 
		),  
		'LastModifiedBy' => array(
			'className' => 'User',  
			'foreignKey' => 'last_modified_by',
			'fields' => array(
				'LastModifiedBy.id',
				'LastModifiedBy.role_id',
				'LastModifiedBy.status',
			) 
		),  
	);
	
}