<?php
App::uses('AppModel', 'Model');
App::uses('SimplePasswordHasher', 'Controller/Component/Auth');
App::uses('Folder', 'Utility');
App::uses('File', 'Utility');

class Ticket extends AppModel { 
	public $useTable = 'support_concerns';
	
	public $validate = array(
        'id' => array(
			'validation 1' => array(
				'required' => 'update',
				'rule' => array('notBlank'),
				'message'	=> 'ID is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'ID must be a non-decimal number that is greater than zero.'
			),
			'validation 3' => array(
				'rule' => array('isUnique'),
				'message'	=> 'ID is already existing.'
			),
			'validation 4' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'Letters and numbers only or non-special characters only.'
			)
        ),
		'support_type' => array(
			'validation 1' => array(
				'rule' => array('maxLength', 20),
				'message'	=> 'Support Type must be no longer than 20 characters.'
			),
        ),
		'ticket_status' => array(
			'validation 1' => array(
				'rule' => array('maxLength', 20),
				'message'	=> 'Ticket Status must be no longer than 20 characters.'
			),
        ), 
		'email_address' => array(
			'validation 1' => array(
				'rule' => array('maxLength', 75),
				'message'	=> 'Email Address must be no longer than 75 characters.',
				'allowEmpty' => true
			),
			'validation 2' => array(
				'rule' => array('email', true),
				'message'	=> 'Please supply a valid Email Address',
				'allowEmpty' => true
			),
		),	
	);
	
	public $belongsTo = array( 
		'TicketType' => array(
			'className' => 'TicketType',
			'foreignKey' => 'concern_type_code',
			'fields' => array( 
				'TicketType.code',
				'TicketType.name'
			)
		),  
		'TicketProduct' => array(
			'className' => 'TicketProduct',
			'foreignKey' => 'product_code',
			'fields' => array( 
				'TicketProduct.code',
				'TicketProduct.product_name'
			)
		),   
	); 
		
	
	public $hasMany = array(
		'TicketHistory' => array(
			'className' => 'TicketHistory',
			'foreignKey' => 'ticket_id',
			'conditions' => array(
				'status' => 'Active'
			),
		),
		 
	);
	 
	public function find_by_id($id = null) { 
		$data = $this->find('first',array(
			'conditions' => array(
				'Ticket.id' => $id,
			), 
			'recursive' => -1
		));
		 
		return $data;
	}
	 
	public function find_by_id_with_contain($id = null) { 
		$data = $this->find('first',array(
			'contain' => array(
				'TicketType',
				'TicketProduct', 
			),
			'conditions' => array(
				'Ticket.id' => $id,
			), 
			'recursive' => -1
		));
		 
		return $data;
	}
	 
	public function beforeSave($options = array()) { 
		if(!isset($this->data['Ticket']['id'])) {
			$this->data['Ticket']['created_by'] = $_SESSION['Auth']['User']['id'];
			$this->data['Ticket']['modified_by'] = $_SESSION['Auth']['User']['id'];
			$this->data['Ticket']['status'] = 'Active';
		}
		else {
			$this->data['Ticket']['modified_by'] = $_SESSION['Auth']['User']['id'];
		}
	 
		return true;
		
	}
	
	public function afterFind($results, $primary = false) {
		foreach($results as $results_key => $results_value) { 
			if (isset($results_value['Ticket']['date'])) {
				$results[$results_key]['Ticket']['date'] = date('m/d/Y', strtotime($results_value['Ticket']['date']));
			}
			 
		} 
		
		return $results;
	}
	 
}
?>