<?php
App::uses('AppModel', 'Model');
App::uses('SimplePasswordHasher', 'Controller/Component/Auth');

class User extends AppModel { 
	public $actsAs = array('DaiSharedFunction', 'DaiSharedCallback');
	
	public $displayField = 'username';
	
	public $audit_model = 'User';
	public $audit_module = 'User';
	public $audit_singular_alias = 'User';
	public $audit_plural_alias = 'Users';
	
	public $validate = array(
        'id' => array(
			'validation 1' => array(
				'required' => 'update',
				'rule' => array('notBlank'),
				'message'	=> 'ID is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'ID must be a non-decimal number that is greater than zero.'
			),
			'validation 3' => array(
				'rule' => array('isUnique'),
				'message'	=> 'ID is already existing.'
			)
        ),
		'username' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Username is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'Username must be in alphabets and numbers only.'
			),
			'validation 3' => array(
				'rule' => array('lengthBetween', 8, 20),
				'message'	=> 'Username must be 8 to 20 characters long.'
			),
			'validation 4' => array(
				'rule' => array('isUnique'),
				'message'	=> 'Username is already taken.'
			)
        ),
		
		'password' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Password is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'Password must be in alphabets and numbers only.'
			),
			'validation 3' => array(
				'rule' => array('lengthBetween', 8, 40),
				'message'	=> 'Password must be 8 to 40 characters long.'
			),
			'validation 4' => array(
				'required' => 'create',
				'on' => 'create',
				'rule' => array('equalToField', 'password', 'confirm_password'),
				'message'	=> 'Password and confirm password does not match.'
			),
			'validation 5' => array(
				'on' => 'update',
				'rule' => array('equalToField', 'password', 'new_password'),
				'message'	=> 'Password and new password does not match.'
			)
        ),
		'confirm_password' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Password must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'Confirm password must be in alphabets and numbers only.'
			),
			'validation 3' => array(
				'rule' => array('lengthBetween', 8, 40),
				'message'	=> 'Confirm password must be 8 to 40 characters long.'
			),
			'validation 4' => array(
				'required' => 'create',
				'on' => 'create',
				'rule' => array('equalToField', 'password', 'confirm_password'),
				'message'	=> 'Password and confirm password does not match.'
			)
        ),
		'current_password' => array(
			'validation 1' => array(
				'rule' => array('notBlank'),
				'message'	=> 'Password must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'Current password must be in alphabets and numbers only.'
			),
			'validation 3' => array(
				'rule' => array('lengthBetween', 8, 40),
				'message'	=> 'Current password must be 8 to 40 characters long.'
			),
			'validation 4' => array(
				'rule' => array('checkIfCurrentPasswordIsCorrect', 'current_password'),
				'message'	=> 'Current password does not match.'
			)
        ),
		'new_password' => array(
			'validation 1' => array(
				'rule' => array('notBlank'),
				'message'	=> 'New password must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'New password must be in alphabets and numbers only.'
			),
			'validation 3' => array(
				'rule' => array('lengthBetween', 8, 40),
				'message'	=> 'New password must be 8 to 40 characters long.'
			),
			'validation 4' => array(
				'rule' => array('equalToField', 'new_password', 'repeat_new_password'),
				'message'	=> 'New password and repeat new password does not match.'
			)
        ),
		'repeat_new_password' => array(
			'validation 1' => array(
				'rule' => array('notBlank'),
				'message'	=> 'Repeat new password must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'Repeat new password must be in alphabets and numbers only.'
			),
			'validation 3' => array(
				'rule' => array('lengthBetween', 8, 40),
				'message'	=> 'Repeat new password must be 8 to 40 characters long.'
			),
			'validation 4' => array(
				'rule' => array('equalToField', 'new_password', 'repeat_new_password'),
				'message'	=> 'Repeat new password and new password does not match.'
			)
        ),
		'role_id' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Role is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'Please supply a valid Role'
			)
        ), 
		'user_privilege_id' => array(
			'validation 1' => array(
				'allowEmpty' => true,
				'rule' => array('naturalNumber'),
				'message'	=> 'Please supply a valid User Priviledge'
			) 
        ),
		'last_name' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Last name is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('maxLength', 50),
				'message'	=> 'Last name must be no longer than 50 characters.'
			),
			'validation 3' => array(
				'rule' => array('isUnique', array('last_name', 'first_name', 'middle_name'), false),
				'message' => 'A combination of Last name, First name and Middle name already exists.'
			)
        ),
		'first_name' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'First name is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('maxLength', 50),
				'message'	=> 'First name must be no longer than 50 characters.'
			),
			'validation 3' => array(
				'rule' => array('isUnique', array('last_name', 'first_name', 'middle_name'), false),
				'message' => 'A combination of Last name, First name and Middle name already exists.'
			)
        ),
		'middle_name' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Middle name is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('maxLength', 50),
				'message'	=> 'Middle name must be no longer than 50 characters.'
			),
			'validation 3' => array(
				'rule' => array('isUnique', array('last_name', 'first_name', 'middle_name'), false),
				'message' => 'A combination of Last name, First name and Middle name already exists.'
			)
        ),
		// 'is_eula_agreed' => array(
			// 'validation 1' => array(
				// 'required' => 'create',
				// 'rule' => array('notBlank'),
				// 'message'	=> 'Eula agreement status must not be empty.'
			// ),
			// 'validation 2' => array(
				// 'rule' => array('boolean'),
				// 'message'	=> 'Eula agreement status must be a boolean.'
			// )
        // ),
		// 'eula_agreed_on_date' => array(
			// 'validation 1' => array(
				// 'on' => 'update',
				// 'rule' => array('datetime', 'ymd'),
				// 'message'	=> 'EULA agreed on date must be in mm/dd/yyyy hh:mm:ss 24 hours format.'
			// ),
			// 'validation 2' => array(
				// 'on' => 'update',
				// 'rule' => array('noFutureDateTime', 'eula_agreed_on_date', 'm/d/Y H:i:s'),
				// 'message'	=> 'EULA agreed on date must not be later than today.'
			// )
        // ),
		'status' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Status is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('boolean'),
				'message'	=> 'Status must be a boolean.'
			)
        ), 
		'created_by' => array(
			'validation 1' => array(
				'required' => 'create',
				'rule' => array('notBlank'),
				'message'	=> 'Created by is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'Please supply a valid Created by' 
			)
        ), 
	);
	 
	
	public $belongsTo = array( 
		'Role' => array(
			'fields' => array(
				'Role.id',
				'Role.name'
			)
		), 
		'CreatedBy' => array(
			'className' => 'User', 
			'foreignKey' => 'created_by', 
		),   
		'LastModifiedBy' => array(
			'className' => 'User', 
			'foreignKey' => 'modified_by', 
		),   
	);
	
	public function checkIfCurrentPasswordIsCorrect($data_for_validation = '', $field_name_to_compare = '') {
		$password_hasher = new SimplePasswordHasher();
		
		$user_id = $this->data[$this->name]['id'];
		$user_hashed_password = $password_hasher->hash($this->data[$this->name][$field_name_to_compare]);
		
		$boolean_result = $this->hasAny(array(
			'User.id' => $user_id,
			'User.password' => $user_hashed_password
		));
		
		return $boolean_result;
    }
	
	public function change_password($data = array()) {
		$data['User']['id'] = $_SESSION['Auth']['User']['id'];

		return $this->save($data);
	}
	
	public function override_password($data = array()) {
		return $this->save($data);
	}
	
	public function check_user_existency($id = null) {
		$boolean_result = $this->hasAny(array(
			'User.id' => $id,
		));
		
		return $boolean_result;
	}
	
	public function find_by_id($id) {
		$results = $this->find('first', array(
			'conditions' => array(
				'User.id' => $id,
			),
			'contain' => array( 
				'Role', 
			)
		));
		
		if(!empty($results)) {
			return $results;
		}
	 
		return false;
	}
	
	public function find_by_id_contain_employee($id = null) {
		$results = $this->find('first', array(
			'conditions' => array(
				'User.id' => $id,
			),
			'contain' => array(
				'Employee'
			)
		));
		
		if(!empty($results)) {
			return $results;
		}
		
		return false;
	}
	
	public function find_by_id_contain_role_owner($id = null) {
		$results = $this->find('first', array(
			'conditions' => array(
				'User.id' => $id,
			),
			'contain' => array(
				'Role',
				'Owner'
			)
		));
		
		if(!empty($results)) {
			return $results;
		}
		
		return false;
	}
	
	public function get_name_of_user($id = null){
		$this->virtualFields = array(
			'combination_whole_name' => 'CONCAT(User.first_name, " ", User.middle_name, " ", User.last_name)',
		);
		
		$result = $this->find('list',array(
			'fields' => array(
				'User.id',
				'User.combination_whole_name',
				'User.first_name',
				'User.middle_name',
				'User.last_name',
			),
			'recursive' => -1
		));
		
		return $result;
		
	}
	public function get_unassigned_users_list($end_user_id = null) {
		$result = $this->find('list', array(
			'fields' => array(
				'User.id',
				'User.username'
			),
			'contain' => array(
				'EndUser'
			),
			'conditions' => array(
				'User.status' => 1,
				'OR' => array(
					array('EndUser.id' => null),
					array('EndUser.id' => $end_user_id)
				)
			),
			'order' => array(
				'User.username ASC'
			),
			'recursive' => -1
		));
		
		return $result;
	}
	
	public function beforeValidate($options = array()) {
		if(!isset($this->data['User']['id'])) {
			$this->data['User']['eula_status'] = 'No';
		}
		else {
			if(isset($this->data['User']['new_password'])) {
				$this->data['User']['password'] = $this->data['User']['new_password'];
			}
		}
	}
	
	public function afterValidate() { 
		if(isset($this->data['User']['password'])) {
			$password_hasher = new SimplePasswordHasher();
			$this->data['User']['password'] = $password_hasher->hash($this->data['User']['password']);
		}
	}
}
