<?php
App::uses('AppModel', 'Model');
App::uses('SimplePasswordHasher', 'Controller/Component/Auth');
App::uses('Folder', 'Utility');
App::uses('File', 'Utility');

class Ticket extends AppModel {
	// public $actsAs = array('AuditLog.Auditable');
	// public $primaryKey = 'id';
	public $useTable = 'support_concerns';
	
	public $validate = array(
        'id' => array(
			'validation 1' => array(
				'required' => 'update',
				'rule' => array('notBlank'),
				'message'	=> 'ID is required and must not be empty.'
			),
			'validation 2' => array(
				'rule' => array('naturalNumber'),
				'message'	=> 'ID must be a non-decimal number that is greater than zero.'
			),
			'validation 3' => array(
				'rule' => array('isUnique'),
				'message'	=> 'ID is already existing.'
			),
			'validation 4' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'Letters and numbers only or non-special characters only.'
			)
        ),
		'support_type' => array(
			'validation 1' => array(
				'rule' => array('maxLength', 20),
				'message'	=> 'Support Type must be no longer than 20 characters.'
			),
        ),
		'ticket_status' => array(
			'validation 1' => array(
				'rule' => array('maxLength', 20),
				'message'	=> 'Ticket Status must be no longer than 20 characters.'
			),
        ),
		'ticket_number' => array(
			'validation 1' => array(
				'rule' => array('alphaNumeric'),
				'message'	=> 'Letters and numbers only or non-special characters only.'
			),
        ),
		'email_address' => array(
			'validation 1' => array(
				'rule' => array('maxLength', 75),
				'message'	=> 'Email Address must be no longer than 75 characters.',
				'allowEmpty' => true
			),
			'validation 2' => array(
				'rule' => array('email', true),
				'message'	=> 'Please supply a valid Email Adress',
				'allowEmpty' => true
			),
		),	
	);

	public $hasMany = array(
		'TicketHistory' => array(
			'className' => 'Ticket',
			'foreignKey' => 'ticket_number',
			'conditions' => array(
				'status' => 'Active'
			),
		),
	);
	
	public function find_all_ticket_history($id=null){
		$data = $this->find('all', array(
			'conditions' => array(
				'Ticket.ticket_number' => $id,
				'Ticket.status' => 'Active'
			),
			'recursive'=> -1
			
		));
		return $data;
	}	
	
	public function find_ticket($id=null){
		$data = $this->find('first', array(
			'conditions' => array(
				'Ticket.id' => $id,
			),
			'recursive'=> -1
			
		));
		return $data;
	}
	
	
	public function close_ticket($id = null){
		$data['Ticket']['id'] = $id;
		$data['Ticket']['ticket_status'] = 'Closed';
		
		if($this->save($data)){
			return true;
		}
		else{
			return false;
		}
	}
	
	
	public function select_ticket_with_id($id = null){
		// $this->primaryKey = 'ticket_number';
		$data = $this->find('first',array(
			'conditions' => array(
				'Ticket.id' => $id,
			),
			'contain' => array(
				'TicketHistory'
			),
			'recursive' => -1
		));
		
		return $data;
	}
	

	public function get_all_tickets(){
		
		$data = $this->find('all',array(
			'order' => 'Ticket.created DESC',
			'recursive' => -1
		));
		
		return $data;	
	}

	public function beforeValidate($options = array()) {

		if(isset($this->data['Ticket']['date']) && validate_datetime($this->data['Ticket']['date'], 'm/d/Y')) {
		}

		
		return true;
	}	
	
	public function beforeSave($options = array()) {
		if(!isset($this->data['Ticket']['id'])) {
			$this->data['Ticket']['created_by'] = $_SESSION['Auth']['User']['id'];
			$this->data['Ticket']['modified_by'] = $_SESSION['Auth']['User']['id'];
			$this->data['Ticket']['status'] = 'Active';
		}
		else {
			$this->data['Ticket']['modified_by'] = $_SESSION['Auth']['User']['id'];
		}
		// debug($this->data);
		// exit();
		return true;
	}
}
?>